@@ -1,7 +1,7 @@
 /*
- * endian.h - handle endianess-problems
+ * endian_handling.h - handle endianess
  *
- * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -26,20 +26,14 @@
 #ifndef _ENDIAN_HANDLING_H
 #define _ENDIAN_HANDLING_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include <QtCore/QSysInfo>
 
-#include "types.h"
+#include "lmms_basics.h"
 
 
-inline bool isLittleEndian( void )
+inline bool isLittleEndian()
 {
-#ifdef WORDS_BIGENDIAN
-	return( FALSE );
-#else
-	return( TRUE );
-#endif
+	return( QSysInfo::ByteOrder == QSysInfo::LittleEndian );
 }
 
 
