@@ -1,7 +1,7 @@
 /*
  * midi.h - constants, structs etc. concerning MIDI
  *
- * Copyright (c) 2005-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2010 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -128,6 +128,11 @@ struct midiEvent
 	{
 	}
 
+	inline MidiEventTypes type() const
+	{
+		return m_type;
+	}
+
 	inline int channel() const
 	{
 		return m_channel;
