@@ -29,19 +29,19 @@
 
 #include <QtGui/QPixmap>
 
-#include "instrument.h"
-#include "instrument_view.h"
+#include "Instrument.h"
+#include "InstrumentView.h"
 #include "sample_buffer.h"
 #include "knob.h"
 #include "pixmap_button.h"
 
 
 
-class audioFileProcessor : public instrument
+class audioFileProcessor : public Instrument
 {
 	Q_OBJECT
 public:
-	audioFileProcessor( instrumentTrack * _instrument_track );
+	audioFileProcessor( InstrumentTrack * _instrument_track );
 	virtual ~audioFileProcessor();
 
 	virtual void playNote( notePlayHandle * _n, 
@@ -54,26 +54,26 @@ public:
 
 	virtual void loadFile( const QString & _file );
 
-	virtual QString nodeName( void ) const;
+	virtual QString nodeName() const;
 
 	virtual Uint32 getBeatLen( notePlayHandle * _n ) const;
 
-	virtual f_cnt_t desiredReleaseFrames( void ) const
+	virtual f_cnt_t desiredReleaseFrames() const
 	{
 		return( 128 );
 	}
 
-	virtual pluginView * instantiateView( QWidget * _parent );
+	virtual PluginView * instantiateView( QWidget * _parent );
 
 
 public slots:
-	void setAudioFile( const QString & _audio_file, bool _rename = TRUE );
+	void setAudioFile( const QString & _audio_file, bool _rename = true );
 
 
 private slots:
-	void reverseModelChanged( void );
-	void ampModelChanged( void );
-	void loopPointChanged( void );
+	void reverseModelChanged();
+	void ampModelChanged();
+	void loopPointChanged();
 
 
 private:
@@ -81,30 +81,30 @@ private:
 
 	sampleBuffer m_sampleBuffer;
 	
-	knobModel m_ampModel;
-	knobModel m_startPointModel;
-	knobModel m_endPointModel;
-	boolModel m_reverseModel;
-	boolModel m_loopModel;
+	FloatModel m_ampModel;
+	FloatModel m_startPointModel;
+	FloatModel m_endPointModel;
+	BoolModel m_reverseModel;
+	BoolModel m_loopModel;
 
 
-	friend class audioFileProcessorView;
+	friend class AudioFileProcessorView;
 
 } ;
 
 
 
-class audioFileProcessorView : public instrumentView
+class AudioFileProcessorView : public InstrumentView
 {
 	Q_OBJECT
 public:
-	audioFileProcessorView( instrument * _instrument, QWidget * _parent );
-	virtual ~audioFileProcessorView();
+	AudioFileProcessorView( Instrument * _instrument, QWidget * _parent );
+	virtual ~AudioFileProcessorView();
 
 
 protected slots:
-	void sampleUpdated( void );
-	void openAudioFile( void );
+	void sampleUpdated();
+	void openAudioFile();
 
 
 protected:
@@ -114,7 +114,7 @@ protected:
 
 
 private:
-	virtual void modelChanged( void );
+	virtual void modelChanged();
 
 	static QPixmap * s_artwork;
 
