@@ -1,7 +1,7 @@
 /*
  * kicker.cpp - bassdrum-synthesizer
  *
- * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -28,19 +28,18 @@
 
 #include "kicker.h"
 #include "engine.h"
-#include "instrument_track.h"
+#include "InstrumentTrack.h"
 #include "knob.h"
 #include "note_play_handle.h"
-#include "sweep_oscillator.h"
+#include "SweepOscillator.h"
 
-#undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
 
 
 extern "C"
 {
 
-plugin::descriptor PLUGIN_EXPORT kicker_plugin_descriptor =
+Plugin::Descriptor PLUGIN_EXPORT kicker_plugin_descriptor =
 {
 	STRINGIFY( PLUGIN_NAME ),
 	"Kicker",
@@ -48,8 +47,8 @@ plugin::descriptor PLUGIN_EXPORT kicker_plugin_descriptor =
 				"versatile kick- & bassdrum-synthesizer" ),
 	"Tobias Doerffel <tobydox/at/users.sf.net>",
 	0x0100,
-	plugin::Instrument,
-	new pluginPixmapLoader( "logo" ),
+	Plugin::Instrument,
+	new PluginPixmapLoader( "logo" ),
 	NULL,
 	NULL
 } ;
@@ -57,8 +56,8 @@ plugin::descriptor PLUGIN_EXPORT kicker_plugin_descriptor =
 }
 
 
-kickerInstrument::kickerInstrument( instrumentTrack * _instrument_track ) :
-	instrument( _instrument_track, &kicker_plugin_descriptor ),
+kickerInstrument::kickerInstrument( InstrumentTrack * _instrument_track ) :
+	Instrument( _instrument_track, &kicker_plugin_descriptor ),
 	m_startFreqModel( 150.0f, 5.0f, 1000.0f, 1.0f, this, tr( "Start frequency" ) ),
 	m_endFreqModel( 40.0f, 5.0f, 1000.0f, 1.0f, this, tr( "End frequency" ) ),
 	m_decayModel( 120.0f, 5.0f, 1000.0f, 1.0f, this, tr( "Decay" ) ),
@@ -102,16 +101,16 @@ void kickerInstrument::loadSettings( const QDomElement & _this )
 
 
 
-QString kickerInstrument::nodeName( void ) const
+QString kickerInstrument::nodeName() const
 {
-	return( kicker_plugin_descriptor.name );
+	return kicker_plugin_descriptor.name;
 }
 
 
 
-//typedef effectLib::foldbackDistortion<> distFX;
-typedef effectLib::distortion distFX;
-typedef sweepOscillator<effectLib::monoToStereoAdaptor<distFX> > sweepOsc;
+//typedef effectLib::foldbackDistortion<> DistFX;
+typedef effectLib::distortion DistFX;
+typedef SweepOscillator<effectLib::monoToStereoAdaptor<DistFX> > SweepOsc;
 
 
 void kickerInstrument::playNote( notePlayHandle * _n,
@@ -123,8 +122,8 @@ void kickerInstrument::playNote( notePlayHandle * _n,
 
 	if ( tfp == 0 )
 	{
-		_n->m_pluginData = new sweepOsc(
-					distFX( m_distModel.value(),
+		_n->m_pluginData = new SweepOsc(
+					DistFX( m_distModel.value(),
 							m_gainModel.value() ) );
 	}
 	else if( tfp > decfr && !_n->released() )
@@ -132,7 +131,7 @@ void kickerInstrument::playNote( notePlayHandle * _n,
 		_n->noteOff();
 	}
 
-	//const float freq = getInstrumentTrack()->frequency( _n ) / 2;
+	//const float freq = instrumentTrack()->frequency( _n ) / 2;
 	const float fdiff = m_endFreqModel.value() - m_startFreqModel.value();
 /*	const fpp_t frames = _n->released() ?
 		tMax( tMin<f_cnt_t>( desiredReleaseFrames() -
@@ -145,7 +144,7 @@ void kickerInstrument::playNote( notePlayHandle * _n,
 	const float f2 = m_startFreqModel.value() + (frames+tfp-1)*fdiff/decfr;
 
 
-	sweepOsc * so = static_cast<sweepOsc *>( _n->m_pluginData );
+	SweepOsc * so = static_cast<SweepOsc *>( _n->m_pluginData );
 	so->update( _working_buffer, frames, f1, f2,
 				engine::getMixer()->processingSampleRate() );
 
@@ -161,7 +160,7 @@ void kickerInstrument::playNote( notePlayHandle * _n,
 		}
 	}
 
-	getInstrumentTrack()->processAudioBuffer( _working_buffer, frames, _n );
+	instrumentTrack()->processAudioBuffer( _working_buffer, frames, _n );
 }
 
 
@@ -169,15 +168,15 @@ void kickerInstrument::playNote( notePlayHandle * _n,
 
 void kickerInstrument::deleteNotePluginData( notePlayHandle * _n )
 {
-	delete static_cast<sweepOsc *>( _n->m_pluginData );
+	delete static_cast<SweepOsc *>( _n->m_pluginData );
 }
 
 
 
 
-pluginView * kickerInstrument::instantiateView( QWidget * _parent )
+PluginView * kickerInstrument::instantiateView( QWidget * _parent )
 {
-	return( new kickerInstrumentView( this, _parent ) );
+	return new kickerInstrumentView( this, _parent );
 }
 
 
@@ -196,9 +195,9 @@ public:
 
 
 
-kickerInstrumentView::kickerInstrumentView( instrument * _instrument,
+kickerInstrumentView::kickerInstrumentView( Instrument * _instrument,
 							QWidget * _parent ) :
-	instrumentView( _instrument, _parent )
+	InstrumentView( _instrument, _parent )
 {
 	m_startFreqKnob = new kickerKnob( this );
 	m_startFreqKnob->setHintText( tr( "Start frequency:" ) + " ", "Hz" );
@@ -220,7 +219,7 @@ kickerInstrumentView::kickerInstrumentView( instrument * _instrument,
 	m_gainKnob->setHintText( tr( "Gain:" ) + " ", "" );
 	m_gainKnob->move( 203, 124 );
 
-	setAutoFillBackground( TRUE );
+	setAutoFillBackground( true );
 	QPalette pal;
 	pal.setBrush( backgroundRole(), PLUGIN_NAME::getIconPixmap(
 								"artwork" ) );
@@ -237,7 +236,7 @@ kickerInstrumentView::~kickerInstrumentView()
 
 
 
-void kickerInstrumentView::modelChanged( void )
+void kickerInstrumentView::modelChanged()
 {
 	kickerInstrument * k = castModel<kickerInstrument>();
 	m_startFreqKnob->setModel( &k->m_startFreqModel );
@@ -254,11 +253,10 @@ void kickerInstrumentView::modelChanged( void )
 extern "C"
 {
 
-// neccessary for getting instance out of shared lib
-plugin * PLUGIN_EXPORT lmms_plugin_main( model *, void * _data )
+// necessary for getting instance out of shared lib
+Plugin * PLUGIN_EXPORT lmms_plugin_main( Model *, void * _data )
 {
-	return( new kickerInstrument(
-				static_cast<instrumentTrack *>( _data ) ) );
+	return new kickerInstrument( static_cast<InstrumentTrack *>( _data ) );
 }
 
 
