@@ -47,7 +47,6 @@ enum parameter_flags
   PF_BOOL = 0x0002,  ///< bool value (usually >=0.5f is treated as TRUE, which is inconsistent with LV2 etc. which treats anything >0 as TRUE)
   PF_ENUM = 0x0003,  ///< enum value (min, min+1, ..., max, only guaranteed to work when min = 0)
   PF_ENUM_MULTI = 0x0004, ///< SET / multiple-choice
-  PF_STRING = 0x0005, ///< see: http://lv2plug.in/docs/index.php?title=String_port
   
   PF_SCALEMASK = 0xF0, ///< bit mask for scale
   PF_SCALE_DEFAULT = 0x00, ///< no scale given
@@ -82,7 +81,6 @@ enum parameter_flags
   PF_PROP_OUTPUT    = 0x080000, ///< output port
   PF_PROP_OPTIONAL  = 0x100000, ///< connection optional
   PF_PROP_GRAPH     = 0x200000, ///< add graph
-  PF_PROP_MSGCONTEXT= 0x400000, ///< message context
   
   PF_UNITMASK     = 0xFF000000,  ///< bit mask for units   \todo reduce to use only 5 bits
   PF_UNIT_DB      = 0x01000000,  ///< decibels
@@ -224,27 +222,15 @@ struct table_column_info
 };
 
 /// 'has string parameters containing tabular data' interface
-struct table_edit_iface
+struct table_metadata_iface
 {
     /// retrieve the table layout for specific parameter
-    virtual const table_column_info *get_table_columns(int param) const = 0;
+    virtual const table_column_info *get_table_columns() const = 0;
 
-    /// return the current number of rows
-    virtual uint32_t get_table_rows(int param) const = 0;
+    /// return the fixed number of rows, or 0 if the number of rows is variable
+    virtual uint32_t get_table_rows() const = 0;
     
-    /// retrieve data item from the plugin
-    virtual std::string get_cell(int param, int row, int column) const;
-
-    /// set data item to the plugin
-    virtual void set_cell(int param, int row, int column, const std::string &src, std::string &error) const { error.clear(); }
-    
-    /// return a line graph interface for a specific parameter/column (unused for now)
-    virtual const line_graph_iface *get_graph_iface(int param, int column) const { return NULL; }
-    
-    /// return an editor name for a specific grid cell (unused for now - I don't even know how editors be implemented)
-    virtual const char *get_cell_editor(int param, int column) const { return NULL; }
-    
-    virtual ~table_edit_iface() {}
+    virtual ~table_metadata_iface() {}
 };
 
 /// 'may receive configure variables' interface
@@ -317,8 +303,6 @@ struct plugin_metadata_iface
     virtual bool requires_midi() const =0;
     /// @return port offset of first control (parameter) port (= number of audio inputs + number of audio outputs in all existing plugins as for 1 Aug 2008)
     virtual int get_param_port_offset() const  = 0;
-    /// @return table_edit_iface if any
-    virtual const table_edit_iface *get_table_edit_iface() const = 0;
     /// @return NULL-terminated list of menu commands
     virtual plugin_command_info *get_commands() const { return NULL; }
     /// @return description structure for given parameter
@@ -331,12 +315,12 @@ struct plugin_metadata_iface
     virtual bool is_cv(int param_no) const = 0;
     /// is the given parameter non-interpolated?
     virtual bool is_noisy(int param_no) const = 0;
-    /// does the plugin require message context? (or DSSI configure) may be slow
-    virtual bool requires_message_context() const = 0;
     /// does the plugin require string port extension? (or DSSI configure) may be slow
-    virtual bool requires_string_ports() const = 0;
-    /// add all message context parameter numbers to the ports vector
-    virtual void get_message_context_parameters(std::vector<int> &ports) const = 0;
+    virtual bool requires_configure() const = 0;
+    /// obtain array of names of configure variables (or NULL is none needed)
+    virtual const char *const *get_configure_vars() const { return NULL; }
+    /// @return table_metadata_iface if any
+    virtual const table_metadata_iface *get_table_metadata_iface(const char *key) const { return NULL; }
 
     /// Do-nothing destructor to silence compiler warning
     virtual ~plugin_metadata_iface() {}
@@ -404,19 +388,19 @@ extern const char *load_gui_xml(const std::string &plugin_id);
 struct audio_module_iface
 {
     /// Handle MIDI Note On
-    virtual void note_on(int note, int velocity) = 0;
+    virtual void note_on(int channel, int note, int velocity) = 0;
     /// Handle MIDI Note Off
-    virtual void note_off(int note, int velocity) = 0;
+    virtual void note_off(int channel, int note, int velocity) = 0;
     /// Handle MIDI Program Change
-    virtual void program_change(int program) = 0;
+    virtual void program_change(int channel, int program) = 0;
     /// Handle MIDI Control Change
-    virtual void control_change(int controller, int value) = 0;
+    virtual void control_change(int channel, int controller, int value) = 0;
     /// Handle MIDI Pitch Bend
     /// @param value pitch bend value (-8192 to 8191, defined as in MIDI ie. 8191 = 200 ct by default)
-    virtual void pitch_bend(int value) = 0;
+    virtual void pitch_bend(int channel, int value) = 0;
     /// Handle MIDI Channel Pressure
     /// @param value channel pressure (0 to 127)
-    virtual void channel_pressure(int value) = 0;
+    virtual void channel_pressure(int channel, int value) = 0;
     /// Called when params are changed (before processing)
     virtual void params_changed() = 0;
     /// LADSPA-esque activate function, except it is called after ports are connected, not before
@@ -427,7 +411,7 @@ struct audio_module_iface
     virtual void set_sample_rate(uint32_t sr) = 0;
     /// Execute menu command with given number
     virtual void execute(int cmd_no) = 0;
-    /// DSSI configure call
+    /// DSSI configure call, value = NULL = reset to default
     virtual char *configure(const char *key, const char *value) = 0;
     /// Send all understood configure vars (none by default)
     virtual void send_configures(send_configure_iface *sci) = 0;
@@ -443,9 +427,9 @@ struct audio_module_iface
     virtual const plugin_metadata_iface *get_metadata_iface() const = 0;
     /// Set the progress report interface to communicate progress to
     virtual void set_progress_report_iface(progress_report_iface *iface) = 0;
-    /// Clear a part of output buffers that have 0s at mask
-    virtual void process_slice(uint32_t offset, uint32_t end) = 0;
-    /// The audio processing loop
+    /// Clear a part of output buffers that have 0s at mask; subdivide the buffer so that no runs > MAX_SAMPLE_RUN are fed to process function
+    virtual uint32_t process_slice(uint32_t offset, uint32_t end) = 0;
+    /// The audio processing loop; assumes numsamples <= MAX_SAMPLE_RUN, for larger buffers, call process_slice
     virtual uint32_t process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask) = 0;
     /// Message port processing function
     virtual uint32_t message_run(const void *valid_ports, void *output_ports) = 0;
@@ -477,19 +461,19 @@ public:
     }
 
     /// Handle MIDI Note On
-    void note_on(int note, int velocity) {}
+    void note_on(int channel, int note, int velocity) {}
     /// Handle MIDI Note Off
-    void note_off(int note, int velocity) {}
+    void note_off(int channel, int note, int velocity) {}
     /// Handle MIDI Program Change
-    void program_change(int program) {}
+    void program_change(int channel, int program) {}
     /// Handle MIDI Control Change
-    void control_change(int controller, int value) {}
+    void control_change(int channel, int controller, int value) {}
     /// Handle MIDI Pitch Bend
     /// @param value pitch bend value (-8192 to 8191, defined as in MIDI ie. 8191 = 200 ct by default)
-    void pitch_bend(int value) {}
+    void pitch_bend(int channel, int value) {}
     /// Handle MIDI Channel Pressure
     /// @param value channel pressure (0 to 127)
-    void channel_pressure(int value) {}
+    void channel_pressure(int channel, int value) {}
     /// Called when params are changed (before processing)
     void params_changed() {}
     /// LADSPA-esque activate function, except it is called after ports are connected, not before
@@ -538,23 +522,23 @@ public:
         }
     }
     /// utility function: call process, and if it returned zeros in output masks, zero out the relevant output port buffers
-    void process_slice(uint32_t offset, uint32_t end)
+    uint32_t process_slice(uint32_t offset, uint32_t end)
     {
+        uint32_t total_out_mask = 0;
         while(offset < end)
         {
             uint32_t newend = std::min(offset + MAX_SAMPLE_RUN, end);
             uint32_t out_mask = process(offset, newend - offset, -1, -1);
+            total_out_mask |= out_mask;
             zero_by_mask(out_mask, offset, newend - offset);
             offset = newend;
         }
+        return total_out_mask;
     }
     /// @return line_graph_iface if any
     virtual const line_graph_iface *get_line_graph_iface() const { return dynamic_cast<const line_graph_iface *>(this); }
 };
 
-extern bool check_for_message_context_ports(const parameter_properties *parameters, int count);
-extern bool check_for_string_ports(const parameter_properties *parameters, int count);
-
 #if USE_EXEC_GUI || USE_DSSI
 
 enum line_graph_item
@@ -616,7 +600,6 @@ public:
     bool get_midi() const { return Metadata::support_midi; }
     bool requires_midi() const { return Metadata::require_midi; }
     bool is_rt_capable() const { return Metadata::rt_capable; }
-    const table_edit_iface *get_table_edit_iface() const { return dynamic_cast<const table_edit_iface *>(this); }    
     int get_param_port_offset()  const { return Metadata::in_count + Metadata::out_count; }
     const char *get_gui_xml() const { static const char *data_ptr = calf_plugins::load_gui_xml(get_id()); return data_ptr; }
     plugin_command_info *get_commands() const { return NULL; }
@@ -625,18 +608,11 @@ public:
     bool is_cv(int param_no) const { return true; }
     bool is_noisy(int param_no) const { return false; }
     const ladspa_plugin_info &get_plugin_info() const { return plugin_info; }
-    bool requires_message_context() const { return check_for_message_context_ports(param_props, Metadata::param_count); }
-    bool requires_string_ports() const { return check_for_string_ports(param_props, Metadata::param_count); }
-    void get_message_context_parameters(std::vector<int> &ports) const {
-        for (int i = 0; i < get_param_count(); ++i) {
-            if (get_param_props(i)->flags & PF_PROP_MSGCONTEXT)
-                ports.push_back(i);
-        }
-    }
+    bool requires_configure() const { return false; }
 };
 
 #define CALF_PORT_NAMES(name) template<> const char *::plugin_metadata<name##_metadata>::port_names[]
-#define CALF_PORT_PROPS(name) template<> parameter_properties plugin_metadata<name##_metadata>::param_props[]
+#define CALF_PORT_PROPS(name) template<> parameter_properties plugin_metadata<name##_metadata>::param_props[name##_metadata::param_count + 1]
 #define CALF_PLUGIN_INFO(name) template<> calf_plugins::ladspa_plugin_info plugin_metadata<name##_metadata>::plugin_info
 #define PLUGIN_NAME_ID_LABEL(name, id, label) \
     static const char *impl_get_name() { return name; } \
@@ -645,22 +621,22 @@ public:
     
 extern const char *calf_copyright_info;
 
-bool get_freq_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context, bool use_frequencies = true);
+bool get_freq_gridline(int subindex, float &pos, bool &vertical, std::string &legend, cairo_iface *context, bool use_frequencies = true, float res = 256, float ofs = 0.4);
     
-/// convert amplitude value to normalized grid-ish value (0dB = 0.5, 30dB = 1.0, -30 dB = 0.0, -60dB = -0.5, -90dB = -1.0)
-static inline float dB_grid(float amp)
+/// convert amplitude value to normalized grid-ish value
+static inline float dB_grid(float amp, float res = 256, float ofs = 0.4)
 {
-    return log(amp) * (1.0 / log(256.0)) + 0.4;
+    return log(amp) * (1.0 / log(res)) + ofs;
 }
 
 template<class Fx>
-static bool get_graph(Fx &fx, int subindex, float *data, int points)
+static bool get_graph(Fx &fx, int subindex, float *data, int points, float res = 256, float ofs = 0.4)
 {
     for (int i = 0; i < points; i++)
     {
         typedef std::complex<double> cfloat;
         double freq = 20.0 * pow (20000.0 / 20.0, i * 1.0 / points);
-        data[i] = dB_grid(fx.freq_gain(subindex, freq, fx.srate));
+        data[i] = dB_grid(fx.freq_gain(subindex, freq, fx.srate), res, ofs);
     }
     return true;
 }
@@ -689,6 +665,59 @@ struct preset_access_iface
     virtual ~preset_access_iface() {} 
 };
 
+/// Implementation of table_metadata_iface providing metadata for mod matrices
+class mod_matrix_metadata: public table_metadata_iface
+{
+public:
+    /// Mapping modes
+    enum mapping_mode {
+        map_positive, ///< 0..100%
+        map_bipolar, ///< -100%..100%
+        map_negative, ///< -100%..0%
+        map_squared, ///< x^2
+        map_squared_bipolar, ///< x^2 scaled to -100%..100%
+        map_antisquared, ///< 1-(1-x)^2 scaled to 0..100%
+        map_antisquared_bipolar, ///< 1-(1-x)^2 scaled to -100..100%
+        map_parabola, ///< inverted parabola (peaks at 0.5, then decreases to 0)
+        map_type_count
+    };
+    const char **mod_src_names, **mod_dest_names;
+
+    mod_matrix_metadata(unsigned int _rows, const char **_src_names, const char **_dest_names);
+    virtual const table_column_info *get_table_columns() const;
+    virtual uint32_t get_table_rows() const;
+
+protected:
+    /// Column descriptions for table widget
+    table_column_info table_columns[6];
+    
+    unsigned int matrix_rows;
+};
+
+/// Check if a given key is either prefix + rows or prefix + i2s(row) + "," + i2s(column)
+/// @arg key key to parse
+/// @arg prefix table prefix (e.g. "modmatrix:")
+/// @arg is_rows[out] set to true if key == prefix + "rows"
+/// @arg row[out] if key is of a form: prefix + row + "," + i2s(column), returns row, otherwise returns -1
+/// @arg column[out] if key is of a form: prefix + row + "," + i2s(column), returns row, otherwise returns -1
+/// @retval true if this is one of the recognized string forms
+extern bool parse_table_key(const char *key, const char *prefix, bool &is_rows, int &row, int &column);
+
+#if USE_EXEC_GUI
+class table_via_configure
+{
+protected:
+    typedef std::pair<int, int> coord;
+    std::vector<table_column_info> columns;
+    std::map<coord, std::string> values;
+    int rows;
+public:
+    table_via_configure();
+    void configure(const char *key, const char *value);
+    virtual ~table_via_configure();
+};
+#endif
+
 };
 
 #endif
