@@ -25,6 +25,9 @@
 
 #include "giface.h"
 
+#define MONO_VU_METER_PARAMS param_meter_in, param_meter_out, param_clip_in, param_clip_out
+#define STEREO_VU_METER_PARAMS param_meter_inL, param_meter_inR, param_meter_outL, param_meter_outR, param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR
+
 namespace calf_plugins {
 
 struct flanger_metadata: public plugin_metadata<flanger_metadata>
@@ -78,7 +81,7 @@ struct vintage_delay_metadata: public plugin_metadata<vintage_delay_metadata>
 struct rotary_speaker_metadata: public plugin_metadata<rotary_speaker_metadata>
 {
 public:
-    enum { par_speed, par_spacing, par_shift, par_moddepth, par_treblespeed, par_bassspeed, par_micdistance, par_reflection, par_meter_l, par_meter_h, param_count };
+    enum { par_speed, par_spacing, par_shift, par_moddepth, par_treblespeed, par_bassspeed, par_micdistance, par_reflection, par_am_depth, par_test, par_meter_l, par_meter_h, param_count };
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = true, require_midi = false, rt_capable = true };
     PLUGIN_NAME_ID_LABEL("rotary_speaker", "rotaryspeaker", "Rotary Speaker")
 };
@@ -115,6 +118,7 @@ struct monosynth_metadata: public plugin_metadata<monosynth_metadata>
         param_count };
     enum { in_count = 0, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = true, require_midi = true, rt_capable = true };
     enum { step_size = 64, step_shift = 6 };
+    enum { mod_matrix_slots = 10 };
     enum {
         modsrc_none,
         modsrc_velocity,
@@ -140,6 +144,13 @@ struct monosynth_metadata: public plugin_metadata<monosynth_metadata>
         moddest_count,
     };
     PLUGIN_NAME_ID_LABEL("monosynth", "monosynth", "Monosynth")
+    
+    mod_matrix_metadata mm_metadata;
+    
+    monosynth_metadata();
+    /// Lookup of table edit interface
+    virtual const table_metadata_iface *get_table_metadata_iface(const char *key) const { if (!strcmp(key, "mod_matrix")) return &mm_metadata; else return NULL; }
+    const char *const *get_configure_vars() const;
 };
 
 /// Thor's compressor - metadata
@@ -147,7 +158,7 @@ struct monosynth_metadata: public plugin_metadata<monosynth_metadata>
 struct compressor_metadata: public plugin_metadata<compressor_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_meter_in, param_meter_out, param_clip_in, param_clip_out,
+    enum { param_bypass, param_level_in, MONO_VU_METER_PARAMS,
            param_threshold, param_ratio, param_attack, param_release, param_makeup, param_knee, param_detection, param_stereo_link, param_compression, 
            param_count };
     PLUGIN_NAME_ID_LABEL("compressor", "compressor", "Compressor")
@@ -157,7 +168,7 @@ struct compressor_metadata: public plugin_metadata<compressor_metadata>
 struct sidechaincompressor_metadata: public plugin_metadata<sidechaincompressor_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_meter_in, param_meter_out, param_clip_in, param_clip_out,
+    enum { param_bypass, param_level_in, MONO_VU_METER_PARAMS,
            param_threshold, param_ratio, param_attack, param_release, param_makeup, param_knee, param_detection, param_stereo_link, param_compression, 
            param_sc_mode, param_f1_freq, param_f2_freq, param_f1_level, param_f2_level,
            param_sc_listen, param_f1_active, param_f2_active, param_count };
@@ -168,8 +179,8 @@ struct sidechaincompressor_metadata: public plugin_metadata<sidechaincompressor_
 struct multibandcompressor_metadata: public plugin_metadata<multibandcompressor_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_meter_inL, param_meter_inR,
-           param_meter_outL, param_meter_outR, param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR,
+    enum { param_bypass, param_level_in, param_level_out, 
+           STEREO_VU_METER_PARAMS,
            param_freq0, param_freq1, param_freq2,
            param_sep0, param_sep1, param_sep2,
            param_q0, param_q1, param_q2,
@@ -197,12 +208,33 @@ struct deesser_metadata: public plugin_metadata<deesser_metadata>
     PLUGIN_NAME_ID_LABEL("deesser", "deesser", "Deesser")
 };
 
+/// Damiens' Gate - metadata
+/// Added some meters and stripped the weighting part
+struct gate_metadata: public plugin_metadata<gate_metadata>
+{
+    enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
+    enum { param_bypass, param_level_in, MONO_VU_METER_PARAMS,
+           param_range, param_threshold, param_ratio, param_attack, param_release, param_makeup, param_knee, param_detection, param_stereo_link, param_gating, 
+           param_count };
+    PLUGIN_NAME_ID_LABEL("gate", "gate", "Gate")
+};
+
+/// Markus's sidechain gate - metadata
+struct sidechaingate_metadata: public plugin_metadata<sidechaingate_metadata>
+{
+    enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
+    enum { param_bypass, param_level_in, MONO_VU_METER_PARAMS,
+           param_range, param_threshold, param_ratio, param_attack, param_release, param_makeup, param_knee, param_detection, param_stereo_link, param_gating, 
+           param_sc_mode, param_f1_freq, param_f2_freq, param_f1_level, param_f2_level,
+           param_sc_listen, param_f1_active, param_f2_active, param_count };
+    PLUGIN_NAME_ID_LABEL("sidechaingate", "sidechaingate", "Sidechain Gate")
+};
+
 /// Markus's 5-band EQ - metadata
 struct equalizer5band_metadata: public plugin_metadata<equalizer5band_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_meter_inL, param_meter_inR,
-           param_meter_outL, param_meter_outR, param_clip_inL, param_clip_outL, param_clip_inR, param_clip_outR,
+    enum { param_bypass, param_level_in, param_level_out, STEREO_VU_METER_PARAMS,
            param_ls_active, param_ls_level, param_ls_freq,
            param_hs_active, param_hs_level, param_hs_freq,
            param_p1_active, param_p1_level, param_p1_freq, param_p1_q,
@@ -219,8 +251,8 @@ struct equalizer5band_metadata: public plugin_metadata<equalizer5band_metadata>
 struct equalizer8band_metadata: public plugin_metadata<equalizer8band_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_meter_inL, param_meter_inR,
-           param_meter_outL, param_meter_outR, param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR,
+    enum { param_bypass, param_level_in, param_level_out, 
+           STEREO_VU_METER_PARAMS,
            param_hp_active, param_hp_freq, param_hp_mode,
            param_lp_active, param_lp_freq, param_lp_mode,
            param_ls_active, param_ls_level, param_ls_freq,
@@ -237,8 +269,8 @@ struct equalizer8band_metadata: public plugin_metadata<equalizer8band_metadata>
 struct equalizer12band_metadata: public plugin_metadata<equalizer12band_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_meter_inL, param_meter_inR,
-           param_meter_outL, param_meter_outR, param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR,
+    enum { param_bypass, param_level_in, param_level_out, 
+           STEREO_VU_METER_PARAMS,
            param_hp_active, param_hp_freq, param_hp_mode,
            param_lp_active, param_lp_freq, param_lp_mode,
            param_ls_active, param_ls_level, param_ls_freq,
@@ -260,8 +292,7 @@ struct equalizer12band_metadata: public plugin_metadata<equalizer12band_metadata
 struct pulsator_metadata: public plugin_metadata<pulsator_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_meter_inL, param_meter_inR,
-           param_meter_outL, param_meter_outR, param_clip_inL, param_clip_inR, param_clip_outL, param_clip_outR,
+    enum { param_bypass, param_level_in, param_level_out, STEREO_VU_METER_PARAMS,
            param_mode, param_freq, param_amount, param_offset, param_mono, param_reset, param_count };
     PLUGIN_NAME_ID_LABEL("pulsator", "pulsator", "Pulsator")
 };
@@ -270,8 +301,7 @@ struct pulsator_metadata: public plugin_metadata<pulsator_metadata>
 struct saturator_metadata: public plugin_metadata<saturator_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 1, outs_optional = 1, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_mix, param_meter_in,
-           param_meter_out, param_clip_in, param_clip_out, param_drive, param_blend, param_meter_drive,
+    enum { param_bypass, param_level_in, param_level_out, param_mix, MONO_VU_METER_PARAMS, param_drive, param_blend, param_meter_drive,
            param_lp_pre_freq, param_hp_pre_freq, param_lp_post_freq, param_hp_post_freq,
            param_p_freq, param_p_level, param_p_q, param_count };
     PLUGIN_NAME_ID_LABEL("saturator", "saturator", "Saturator")
@@ -280,8 +310,7 @@ struct saturator_metadata: public plugin_metadata<saturator_metadata>
 struct exciter_metadata: public plugin_metadata<exciter_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 1, outs_optional = 1, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_amount, param_meter_in,
-           param_meter_out, param_clip_in, param_clip_out, param_drive, param_blend, param_meter_drive,
+    enum { param_bypass, param_level_in, param_level_out, param_amount, MONO_VU_METER_PARAMS, param_drive, param_blend, param_meter_drive,
            param_freq, param_listen, param_count };
     PLUGIN_NAME_ID_LABEL("exciter", "exciter", "Exciter")
 };
@@ -289,21 +318,11 @@ struct exciter_metadata: public plugin_metadata<exciter_metadata>
 struct bassenhancer_metadata: public plugin_metadata<bassenhancer_metadata>
 {
     enum { in_count = 2, out_count = 2, ins_optional = 1, outs_optional = 1, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_bypass, param_level_in, param_level_out, param_amount, param_meter_in,
-           param_meter_out, param_clip_in, param_clip_out, param_drive, param_blend, param_meter_drive,
+    enum { param_bypass, param_level_in, param_level_out, param_amount, MONO_VU_METER_PARAMS, param_drive, param_blend, param_meter_drive,
            param_freq, param_listen, param_count };
     PLUGIN_NAME_ID_LABEL("bassenhancer", "bassenhancer", "Bass Enhancer")
 };
 
-/// Damien's gate - metadata
-struct gate_metadata: public plugin_metadata<gate_metadata>
-{
-    enum { in_count = 3, out_count = 2, ins_optional = 1, outs_optional = 0, support_midi = false, require_midi = false, rt_capable = true };
-    enum { param_threshold, param_ratio, param_attack, param_release, param_makeup, param_knee, param_detection, param_stereo_link, param_aweighting, param_gating, param_peak, param_clip, param_bypass, param_range, param_mono, param_trigger, // param_freq, param_bw,
-        param_count };
-    PLUGIN_NAME_ID_LABEL("gate", "gate", "Gate")
-};
-
 /// Organ - enums for parameter IDs etc. (this mess is caused by organ split between plugin and generic class - which was
 /// a bad design decision and should be sorted out some day) XXXKF @todo
 struct organ_enums
@@ -328,7 +347,7 @@ struct organ_enums
         par_eg1attack, par_eg1decay, par_eg1sustain, par_eg1release, par_eg1velscl, par_eg1ampctl, 
         par_eg2attack, par_eg2decay, par_eg2sustain, par_eg2release, par_eg2velscl, par_eg2ampctl, 
         par_eg3attack, par_eg3decay, par_eg3sustain, par_eg3release, par_eg3velscl, par_eg3ampctl, 
-        par_lforate, par_lfoamt, par_lfowet, par_lfophase, par_lfomode,
+        par_lforate, par_lfoamt, par_lfowet, par_lfophase, par_lfomode, par_lfotype,
         par_transpose, par_detune,
         par_polyphony,
         par_quadenv,
@@ -337,12 +356,8 @@ struct organ_enums
         par_bassgain,
         par_treblefreq,
         par_treblegain,
-        par_var_mapcurve,
         param_count
     };
-    enum {
-        var_count = 1
-    };
     enum organ_waveform { 
         wave_sine, 
         wave_sinepl1, wave_sinepl2, wave_sinepl3,
@@ -369,6 +384,14 @@ struct organ_enums
         ampctl_all,
         ampctl_count
     };
+    enum { 
+        lfotype_allpass = 0,
+        lfotype_cv1,
+        lfotype_cv2,
+        lfotype_cv3,
+        lfotype_cvfull,
+        lfotype_count
+    };
     enum { 
         lfomode_off = 0,
         lfomode_direct,
@@ -392,15 +415,21 @@ struct organ_metadata: public organ_enums, public plugin_metadata<organ_metadata
 {
     enum { in_count = 0, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = true, require_midi = true, rt_capable = true };
     PLUGIN_NAME_ID_LABEL("organ", "organ", "Organ")
+    
+public:
     plugin_command_info *get_commands();
+    const char *const *get_configure_vars() const;
 };
 
 /// FluidSynth - metadata
 struct fluidsynth_metadata: public plugin_metadata<fluidsynth_metadata>
 {
-    enum { par_master, par_interpolation, par_reverb, par_chorus, par_soundfont, par_preset_key_set, param_count };
+    enum { par_master, par_interpolation, par_reverb, par_chorus, param_count };
     enum { in_count = 0, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = true, require_midi = true, rt_capable = false };
     PLUGIN_NAME_ID_LABEL("fluidsynth", "fluidsynth", "Fluidsynth")
+
+public:
+    const char *const *get_configure_vars() const;
 };
     
 /// Wavetable - metadata
@@ -469,8 +498,14 @@ struct wavetable_metadata: public plugin_metadata<wavetable_metadata>
         par_pwhlrange, 
         param_count };
     enum { in_count = 0, out_count = 2, ins_optional = 0, outs_optional = 0, support_midi = true, require_midi = true, rt_capable = true };
+    enum { mod_matrix_slots = 10 };
     enum { step_size = 64 };
     PLUGIN_NAME_ID_LABEL("wavetable", "wavetable", "Wavetable")
+    mod_matrix_metadata mm_metadata;
+    
+    wavetable_metadata();
+    /// Lookup of table edit interface
+    virtual const table_metadata_iface *get_table_metadata_iface(const char *key) const { if (!strcmp(key, "mod_matrix")) return &mm_metadata; else return NULL; }
 };
     
 };
