@@ -26,8 +26,6 @@
 using namespace dsp;
 using namespace calf_plugins;
 
-#define SET_IF_CONNECTED(name) if (params[AM::param_##name] != NULL) *params[AM::param_##name] = name;
-
 /// Saturator Band by Markus Schmidt
 ///
 /// This module is based on Krzysztof's filters and Tom Szilagyi's distortion routine.
@@ -38,10 +36,6 @@ saturator_audio_module::saturator_audio_module()
 {
     is_active = false;
     srate = 0;
-    clip_in    = 0.f;
-    clip_out   = 0.f;
-    meter_in  = 0.f;
-    meter_out = 0.f;
     meter_drive = 0.f;
 }
 
@@ -50,6 +44,8 @@ void saturator_audio_module::activate()
     is_active = true;
     // set all filters
     params_changed();
+    meters.reset();
+    meter_drive = 0.f;
 }
 void saturator_audio_module::deactivate()
 {
@@ -115,11 +111,14 @@ void saturator_audio_module::set_sample_rate(uint32_t sr)
     dist[0].set_sample_rate(sr);
     if(in_count > 1 && out_count > 1)
         dist[1].set_sample_rate(sr);
+    meters.set_sample_rate(srate);
 }
 
 uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
 {
     bool bypass = *params[param_bypass] > 0.5f;
+    uint32_t orig_offset = offset;
+    uint32_t orig_numsamples = numsamples;
     numsamples += offset;
     if(bypass) {
         // everything bypassed
@@ -137,18 +136,8 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
             }
             ++offset;
         }
-        // displays, too
-        clip_in    = 0.f;
-        clip_out   = 0.f;
-        meter_in  = 0.f;
-        meter_out = 0.f;
-        meter_drive = 0.f;
+        meters.bypassed(params, orig_numsamples);
     } else {
-        
-        clip_in    -= std::min(clip_in,  numsamples);
-        clip_out   -= std::min(clip_out, numsamples);
-        meter_in = 0.f;
-        meter_out = 0.f;
         meter_drive = 0.f;
         float in_avg[2] = {0.f, 0.f};
         float out_avg[2] = {0.f, 0.f};
@@ -157,7 +146,6 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
         while(offset < numsamples) {
             // cycle through samples
             float out[2], in[2] = {0.f, 0.f};
-            float maxIn, maxOut = 0.f;
             int c = 0;
             
             if(in_count > 1 && out_count > 1) {
@@ -178,6 +166,8 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
             proc[0] = in[0] * *params[param_level_in];
             proc[1] = in[1] * *params[param_level_in];
             
+            float onedivlevelin = 1.0 / *params[param_level_in];
+            
             for (int i = 0; i < c; ++i) {
                 // all pre filters in chain
                 proc[i] = lp[i][1].process(lp[i][0].process(proc[i]));
@@ -200,7 +190,7 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
                 proc[i] = hp[i][2].process(hp[i][3].process(proc[i]));
                 
                 //subtract gain
-                 proc[i] /= *params[param_level_in];
+                proc[i] *= onedivlevelin;
             }
             
             if(in_count > 1 && out_count > 1) {
@@ -209,42 +199,23 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
                 outs[0][offset] = out[0];
                 out[1] = ((proc[1] * *params[param_mix]) + in[1] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[1][offset] = out[1];
-                maxIn = std::max(fabs(in[0]), fabs(in[1]));
-                maxOut = std::max(fabs(out[0]), fabs(out[1]));
             } else if(out_count > 1) {
                 // mono -> pseudo stereo
                 out[0] = ((proc[0] * *params[param_mix]) + in[0] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[0][offset] = out[0];
                 out[1] = out[0];
                 outs[1][offset] = out[1];
-                maxOut = fabs(out[0]);
-                maxIn = fabs(in[0]);
             } else {
                 // stereo -> mono
                 // or full mono
                 out[0] = ((proc[0] * *params[param_mix]) + in[0] * (1 - *params[param_mix])) * *params[param_level_out];
                 outs[0][offset] = out[0];
-                maxIn = fabs(in[0]);
-                maxOut = fabs(out[0]);
             }
-            
-            if(maxIn > 1.f) {
-                clip_in  = srate >> 3;
-            }
-            if(maxOut > 1.f) {
-                clip_out = srate >> 3;
-            }
-            // set up in / out meters
-            if(maxIn > meter_in) {
-                meter_in = maxIn;
-            }
-            if(maxOut > meter_out) {
-                meter_out = maxOut;
-            }
-            
+                        
             // next sample
             ++offset;
         } // cycle trough samples
+        meters.process(params, ins, outs, orig_offset, orig_numsamples);
         
         tube_avg = (sqrt(std::max(out_avg[0], out_avg[1])) / numsamples) - (sqrt(std::max(in_avg[0], in_avg[1])) / numsamples);
         meter_drive = (5.0f * fabs(tube_avg) * (float(*params[param_blend]) + 30.0f));
@@ -270,19 +241,6 @@ uint32_t saturator_audio_module::process(uint32_t offset, uint32_t numsamples, u
         p[1].sanitize();
     }
     // draw meters
-    if(params[param_clip_in] != NULL) {
-        *params[param_clip_in] = clip_in;
-    }
-    if(params[param_clip_out] != NULL) {
-        *params[param_clip_out] = clip_out;
-    }
-    
-    if(params[param_meter_in] != NULL) {
-        *params[param_meter_in] = meter_in;
-    }
-    if(params[param_meter_out] != NULL) {
-        *params[param_meter_out] = meter_out;
-    }
     if(params[param_meter_drive] != NULL) {
         *params[param_meter_drive] = meter_drive;
     }
@@ -300,10 +258,6 @@ exciter_audio_module::exciter_audio_module()
 {
     is_active = false;
     srate = 0;
-    clip_in    = 0.f;
-    clip_out   = 0.f;
-    meter_in  = 0.f;
-    meter_out = 0.f;
     meter_drive = 0.f;
 }
 
@@ -347,10 +301,13 @@ void exciter_audio_module::set_sample_rate(uint32_t sr)
     dist[0].set_sample_rate(sr);
     if(in_count > 1 && out_count > 1)
         dist[1].set_sample_rate(sr);
+    meters.set_sample_rate(srate);
 }
 
 uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
 {
+    uint32_t orig_offset = offset;
+    uint32_t orig_numsamples = numsamples;
     bool bypass = *params[param_bypass] > 0.5f;
     numsamples += offset;
     if(bypass) {
@@ -369,25 +326,20 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
             }
             ++offset;
         }
+        meters.bypassed(params, orig_numsamples);
         // displays, too
-        clip_in    = 0.f;
-        clip_out   = 0.f;
-        meter_in  = 0.f;
-        meter_out = 0.f;
         meter_drive = 0.f;
     } else {
         
-        clip_in    -= std::min(clip_in,  numsamples);
-        clip_out   -= std::min(clip_out, numsamples);
-        meter_in = 0.f;
-        meter_out = 0.f;
         meter_drive = 0.f;
         
+        float in2out = *params[param_listen] > 0.f ? 0.f : 1.f;
+        
         // process
         while(offset < numsamples) {
             // cycle through samples
             float out[2], in[2] = {0.f, 0.f};
-            float maxIn, maxOut, maxDrive = 0.f;
+            float maxDrive = 0.f;
             int c = 0;
             
             if(in_count > 1 && out_count > 1) {
@@ -421,61 +373,28 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
                 // all post filters in chain
                 proc[i] = hp[i][2].process(hp[i][3].process(proc[i]));
             }
+            maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             
             if(in_count > 1 && out_count > 1) {
+                maxDrive = std::max(maxDrive, dist[1].get_distortion_level() * *params[param_amount]);
                 // full stereo
-                if(*params[param_listen] > 0.f)
-                    out[0] = proc[0] * *params[param_amount] * *params[param_level_out];
-                else
-                    out[0] = (proc[0] * *params[param_amount] + in[0]) * *params[param_level_out];
+                out[0] = (proc[0] * *params[param_amount] + in2out * in[0]) * *params[param_level_out];
+                out[1] = (proc[1] * *params[param_amount] + in2out * in[1]) * *params[param_level_out];
                 outs[0][offset] = out[0];
-                if(*params[param_listen] > 0.f)
-                    out[1] = proc[1] * *params[param_amount] * *params[param_level_out];
-                else
-                    out[1] = (proc[1] * *params[param_amount] + in[1]) * *params[param_level_out];
                 outs[1][offset] = out[1];
-                maxIn = std::max(fabs(in[0]), fabs(in[1]));
-                maxOut = std::max(fabs(out[0]), fabs(out[1]));
-                maxDrive = std::max(dist[0].get_distortion_level() * *params[param_amount],
-                                            dist[1].get_distortion_level() * *params[param_amount]);
             } else if(out_count > 1) {
                 // mono -> pseudo stereo
-                if(*params[param_listen] > 0.f)
-                    out[0] = proc[0] * *params[param_amount] * *params[param_level_out];
-                else
-                    out[0] = (proc[0] * *params[param_amount] + in[0]) * *params[param_level_out];
+                out[1] = out[0] = (proc[0] * *params[param_amount] + in2out * in[0]) * *params[param_level_out];
                 outs[0][offset] = out[0];
-                out[1] = out[0];
                 outs[1][offset] = out[1];
-                maxOut = fabs(out[0]);
-                maxIn = fabs(in[0]);
-                maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             } else {
                 // stereo -> mono
                 // or full mono
-                if(*params[param_listen] > 0.f)
-                    out[0] = proc[0] * *params[param_amount] * *params[param_level_out];
-                else
-                    out[0] = (proc[0] * *params[param_amount] + in[0]) * *params[param_level_out];
+                out[0] = (proc[0] * *params[param_amount] + in2out * in[0]) * *params[param_level_out];
                 outs[0][offset] = out[0];
-                maxIn = fabs(in[0]);
-                maxOut = fabs(out[0]);
-                maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             }
             
-            if(maxIn > 1.f) {
-                clip_in  = srate >> 3;
-            }
-            if(maxOut > 1.f) {
-                clip_out = srate >> 3;
-            }
             // set up in / out meters
-            if(maxIn > meter_in) {
-                meter_in = maxIn;
-            }
-            if(maxOut > meter_out) {
-                meter_out = maxOut;
-            }
             if(maxDrive > meter_drive) {
                 meter_drive = maxDrive;
             }
@@ -483,6 +402,7 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
             // next sample
             ++offset;
         } // cycle trough samples
+        meters.process(params, ins, outs, orig_offset, orig_numsamples);
         // clean up
         hp[0][0].sanitize();
         hp[1][0].sanitize();
@@ -494,19 +414,6 @@ uint32_t exciter_audio_module::process(uint32_t offset, uint32_t numsamples, uin
         hp[1][3].sanitize();
     }
     // draw meters
-    if(params[param_clip_in] != NULL) {
-        *params[param_clip_in] = clip_in;
-    }
-    if(params[param_clip_out] != NULL) {
-        *params[param_clip_out] = clip_out;
-    }
-    
-    if(params[param_meter_in] != NULL) {
-        *params[param_meter_in] = meter_in;
-    }
-    if(params[param_meter_out] != NULL) {
-        *params[param_meter_out] = meter_out;
-    }
     if(params[param_meter_drive] != NULL) {
         *params[param_meter_drive] = meter_drive;
     }
@@ -524,16 +431,14 @@ bassenhancer_audio_module::bassenhancer_audio_module()
 {
     is_active = false;
     srate = 0;
-    clip_in    = 0.f;
-    clip_out   = 0.f;
-    meter_in  = 0.f;
-    meter_out = 0.f;
+    meters.reset();
     meter_drive = 0.f;
 }
 
 void bassenhancer_audio_module::activate()
 {
     is_active = true;
+    meters.reset();
     // set all filters
     params_changed();
 }
@@ -570,11 +475,14 @@ void bassenhancer_audio_module::set_sample_rate(uint32_t sr)
     dist[0].set_sample_rate(sr);
     if(in_count > 1 && out_count > 1)
         dist[1].set_sample_rate(sr);
+    meters.set_sample_rate(srate);
 }
 
 uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples, uint32_t inputs_mask, uint32_t outputs_mask)
 {
     bool bypass = *params[param_bypass] > 0.5f;
+    uint32_t orig_offset = offset;
+    uint32_t orig_numsamples = numsamples;
     numsamples += offset;
     if(bypass) {
         // everything bypassed
@@ -593,24 +501,16 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
             ++offset;
         }
         // displays, too
-        clip_in    = 0.f;
-        clip_out   = 0.f;
-        meter_in  = 0.f;
-        meter_out = 0.f;
+        meters.bypassed(params, orig_numsamples);
         meter_drive = 0.f;
     } else {
-        
-        clip_in    -= std::min(clip_in,  numsamples);
-        clip_out   -= std::min(clip_out, numsamples);
-        meter_in = 0.f;
-        meter_out = 0.f;
         meter_drive = 0.f;
         
         // process
         while(offset < numsamples) {
             // cycle through samples
             float out[2], in[2] = {0.f, 0.f};
-            float maxIn, maxOut, maxDrive = 0.f;
+            float maxDrive = 0.f;
             int c = 0;
             
             if(in_count > 1 && out_count > 1) {
@@ -657,8 +557,6 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
                 else
                     out[1] = (proc[1] * *params[param_amount] + in[1]) * *params[param_level_out];
                 outs[1][offset] = out[1];
-                maxIn = std::max(fabs(in[0]), fabs(in[1]));
-                maxOut = std::max(fabs(out[0]), fabs(out[1]));
                 maxDrive = std::max(dist[0].get_distortion_level() * *params[param_amount],
                                             dist[1].get_distortion_level() * *params[param_amount]);
             } else if(out_count > 1) {
@@ -670,8 +568,6 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
                 outs[0][offset] = out[0];
                 out[1] = out[0];
                 outs[1][offset] = out[1];
-                maxOut = fabs(out[0]);
-                maxIn = fabs(in[0]);
                 maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             } else {
                 // stereo -> mono
@@ -681,24 +577,10 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
                 else
                     out[0] = (proc[0] * *params[param_amount] + in[0]) * *params[param_level_out];
                 outs[0][offset] = out[0];
-                maxIn = fabs(in[0]);
-                maxOut = fabs(out[0]);
                 maxDrive = dist[0].get_distortion_level() * *params[param_amount];
             }
             
-            if(maxIn > 1.f) {
-                clip_in  = srate >> 3;
-            }
-            if(maxOut > 1.f) {
-                clip_out = srate >> 3;
-            }
             // set up in / out meters
-            if(maxIn > meter_in) {
-                meter_in = maxIn;
-            }
-            if(maxOut > meter_out) {
-                meter_out = maxOut;
-            }
             if(maxDrive > meter_drive) {
                 meter_drive = maxDrive;
             }
@@ -706,6 +588,7 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
             // next sample
             ++offset;
         } // cycle trough samples
+        meters.process(params, ins, outs, orig_offset, orig_numsamples);
         // clean up
         lp[0][0].sanitize();
         lp[1][0].sanitize();
@@ -717,19 +600,6 @@ uint32_t bassenhancer_audio_module::process(uint32_t offset, uint32_t numsamples
         lp[1][3].sanitize();
     }
     // draw meters
-    if(params[param_clip_in] != NULL) {
-        *params[param_clip_in] = clip_in;
-    }
-    if(params[param_clip_out] != NULL) {
-        *params[param_clip_out] = clip_out;
-    }
-    
-    if(params[param_meter_in] != NULL) {
-        *params[param_meter_in] = meter_in;
-    }
-    if(params[param_meter_out] != NULL) {
-        *params[param_meter_out] = meter_out;
-    }
     if(params[param_meter_drive] != NULL) {
         *params[param_meter_drive] = meter_drive;
     }
