@@ -27,34 +27,8 @@ using namespace std;
 
 float silence[4097];
 
-static const char *monosynth_mod_src_names[] = {
-    "None", 
-    "Velocity",
-    "Pressure",
-    "ModWheel",
-    "Envelope 1",
-    "Envelope 2",
-    "LFO 1",
-    "LFO 2",
-    NULL
-};
-
-static const char *monosynth_mod_dest_names[] = {
-    "None",
-    "Attenuation",
-    "Osc Mix Ratio (%)",
-    "Cutoff [ct]",
-    "Resonance",
-    "O1: Detune [ct]",
-    "O2: Detune [ct]",
-    "O1: PW (%)",
-    "O2: PW (%)",
-    "O1: Stretch",
-    NULL
-};
-
 monosynth_audio_module::monosynth_audio_module()
-: mod_matrix(mod_matrix_data, mod_matrix_slots, monosynth_mod_src_names, monosynth_mod_dest_names)
+: mod_matrix_impl(mod_matrix_data, &mm_metadata)
 , inertia_cutoff(1)
 , inertia_pitchbend(1)
 , inertia_pressure(64)
@@ -76,6 +50,7 @@ void monosynth_audio_module::activate() {
     stack.clear();
     last_pwshift1 = last_pwshift2 = 0;
     last_stretch1 = 65536;
+    queue_note_on_and_off = false;
 }
 
 waveform_family<MONOSYNTH_WAVE_BITS> *monosynth_audio_module::waves;
@@ -433,6 +408,12 @@ void monosynth_audio_module::delayed_note_on()
     queue_note_on = -1;
     float modsrc[modsrc_count] = { 1, velocity, inertia_pressure.get_last(), modwheel_value, envelope1.value, envelope2.value, 0.5+0.5*lfo1.last, 0.5+0.5*lfo2.last};
     calculate_modmatrix(moddest, moddest_count, modsrc);
+        
+    if (queue_note_on_and_off)
+    {
+        end_note();
+        queue_note_on_and_off = false;
+    }
 }
 
 void monosynth_audio_module::set_sample_rate(uint32_t sr) {
@@ -626,46 +607,58 @@ void monosynth_audio_module::apply_fadeout()
     }
 }
 
-void monosynth_audio_module::note_on(int note, int vel)
+void monosynth_audio_module::note_on(int /*channel*/, int note, int vel)
 {
     queue_note_on = note;
+    queue_note_on_and_off = false;
     last_key = note;
     queue_vel = vel / 127.f;
     stack.push(note);
 }
 
-void monosynth_audio_module::note_off(int note, int vel)
+void monosynth_audio_module::note_off(int /*channel*/, int note, int vel)
 {
     stack.pop(note);
+    if (note == queue_note_on)
+    {
+        queue_note_on_and_off = true;
+        return;
+    }
     // If releasing the currently played note, try to get another one from note stack.
     if (note == last_key) {
-        if (stack.count())
-        {
-            last_key = note = stack.nth(stack.count() - 1);
-            start_freq = freq;
-            target_freq = freq = dsp::note_to_hz(note);
-            porta_time = 0;
-            set_frequency();
-            if (!(legato & 1)) {
-                envelope1.note_on();
-                envelope2.note_on();
-                stopping = false;
-                running = true;
-            }
-            return;
-        }
-        gate = false;
-        envelope1.note_off();
-        envelope2.note_off();
+        end_note();
     }
 }
 
-void monosynth_audio_module::channel_pressure(int value)
+void monosynth_audio_module::end_note()
+{
+    if (stack.count())
+    {
+        int note;
+        last_key = note = stack.nth(stack.count() - 1);
+        start_freq = freq;
+        target_freq = freq = dsp::note_to_hz(note);
+        porta_time = 0;
+        set_frequency();
+        if (!(legato & 1)) {
+            envelope1.note_on();
+            envelope2.note_on();
+            stopping = false;
+            running = true;
+        }
+        return;
+    }
+    gate = false;
+    envelope1.note_off();
+    envelope2.note_off();
+}
+
+void monosynth_audio_module::channel_pressure(int /*channel*/, int value)
 {
     inertia_pressure.set_inertia(value * (1.0 / 127.0));
 }
 
-void monosynth_audio_module::control_change(int controller, int value)
+void monosynth_audio_module::control_change(int /*channel*/, int controller, int value)
 {
     switch(controller)
     {
