@@ -26,18 +26,19 @@
 
 #include <QtGui/QPainter>
 #include <QtXml/QDomElement>
+
+#include <cstdio>
+
 #include "sid.h"
 
 #include "sid_instrument.h"
 #include "engine.h"
-#include "instrument_track.h"
+#include "InstrumentTrack.h"
 #include "knob.h"
 #include "note_play_handle.h"
-#include "sweep_oscillator.h"
 #include "pixmap_button.h"
 #include "tooltip.h"
 
-#undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
 
 #define C64_PAL_CYCLES_PER_SEC  985248
@@ -67,7 +68,7 @@ static const int relTime[16] = { 6, 24, 48, 72, 114, 168, 204, 240, 300, 750,
 
 extern "C"
 {
-plugin::descriptor PLUGIN_EXPORT sid_plugin_descriptor =
+Plugin::Descriptor PLUGIN_EXPORT sid_plugin_descriptor =
 {
 	STRINGIFY( PLUGIN_NAME ),
 	"SID",
@@ -77,16 +78,16 @@ plugin::descriptor PLUGIN_EXPORT sid_plugin_descriptor =
 	"Csaba Hruska <csaba.hruska/at/gmail.com>"
 	"Attila Herman <attila589/at/gmail.com>",
 	0x0100,
-	plugin::Instrument,
-	new pluginPixmapLoader( "logo" ),
+	Plugin::Instrument,
+	new PluginPixmapLoader( "logo" ),
 	NULL,
 	NULL
 } ;
 
 }
 
-voiceObject::voiceObject( model * _parent, int _idx ) :
-	model( _parent ),
+voiceObject::voiceObject( Model * _parent, int _idx ) :
+	Model( _parent ),
 	m_pulseWidthModel( 2048.0f, 0.0f, 4095.0f, 1.0f, this,
 					tr( "Voice %1 pulse width" ).arg( _idx+1 ) ),
 	m_attackModel( 8.0f, 0.0f, 15.0f, 1.0f, this,
@@ -115,8 +116,8 @@ voiceObject::~voiceObject()
 }
 
 
-sidInstrument::sidInstrument( instrumentTrack * _instrument_track ) :
-	instrument( _instrument_track, &sid_plugin_descriptor ),
+sidInstrument::sidInstrument( InstrumentTrack * _instrument_track ) :
+	Instrument( _instrument_track, &sid_plugin_descriptor ),
 	// filter	
 	m_filterFCModel( 1024.0f, 0.0f, 2047.0f, 1.0f, this, tr( "Cutoff" ) ),
 	m_filterResonanceModel( 8.0f, 0.0f, 15.0f, 1.0f, this, tr( "Resonance" ) ),
@@ -216,7 +217,7 @@ void sidInstrument::loadSettings( const QDomElement & _this )
 
 
 
-QString sidInstrument::nodeName( void ) const
+QString sidInstrument::nodeName() const
 {
 	return( sid_plugin_descriptor.name );
 }
@@ -224,7 +225,7 @@ QString sidInstrument::nodeName( void ) const
 
 
 
-f_cnt_t sidInstrument::desiredReleaseFrames( void ) const
+f_cnt_t sidInstrument::desiredReleaseFrames() const
 {
 	const float samplerate = engine::getMixer()->processingSampleRate();
 	int maxrel = 0;
@@ -430,7 +431,7 @@ void sidInstrument::playNote( notePlayHandle * _n,
 		}
 	}
 
-	getInstrumentTrack()->processAudioBuffer( _working_buffer, frames, _n );
+	instrumentTrack()->processAudioBuffer( _working_buffer, frames, _n );
 }
 
 
@@ -444,7 +445,7 @@ void sidInstrument::deleteNotePluginData( notePlayHandle * _n )
 
 
 
-pluginView * sidInstrument::instantiateView( QWidget * _parent )
+PluginView * sidInstrument::instantiateView( QWidget * _parent )
 {
 	return( new sidInstrumentView( this, _parent ) );
 }
@@ -471,12 +472,12 @@ public:
 
 
 
-sidInstrumentView::sidInstrumentView( instrument * _instrument,
+sidInstrumentView::sidInstrumentView( Instrument * _instrument,
 							QWidget * _parent ) :
-	instrumentView( _instrument, _parent )
+	InstrumentView( _instrument, _parent )
 {
 
-	setAutoFillBackground( TRUE );
+	setAutoFillBackground( true );
 	QPalette pal;
 	pal.setBrush( backgroundRole(), PLUGIN_NAME::getIconPixmap( "artwork" ) );
 	setPalette( pal );
@@ -517,7 +518,7 @@ sidInstrumentView::sidInstrumentView( instrument * _instrument,
 	m_passBtnGrp->addButton( lp_btn );
 
 	m_offButton = new pixmapButton( this, NULL );
-	m_offButton->setCheckable( TRUE );
+	m_offButton->setCheckable( true );
 	m_offButton->move( 176 + 3*14, 53 );
 	m_offButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap( "3offred" ) );
 	m_offButton->setInactiveGraphic( PLUGIN_NAME::getIconPixmap( "3off" ) );
@@ -623,7 +624,7 @@ sidInstrumentView::sidInstrumentView( instrument * _instrument,
 		int syncRingWidth[] = { 3, 1, 2 };
 
 		pixmapButton * sync_btn = new pixmapButton( this, NULL );
-		sync_btn->setCheckable( TRUE );
+		sync_btn->setCheckable( true );
 		sync_btn->move( 191, 117 + i*50 );
 		sync_btn->setActiveGraphic(
 			PLUGIN_NAME::getIconPixmap( "syncred" ) );
@@ -636,7 +637,7 @@ sidInstrumentView::sidInstrumentView( instrument * _instrument,
 			.arg( syncRingWidth[i] ) );
 
 		pixmapButton * ringMod_btn = new pixmapButton( this, NULL );
-		ringMod_btn->setCheckable( TRUE );
+		ringMod_btn->setCheckable( true );
 		ringMod_btn->move( 191 + 2*14, 117 + i*50 );
 		ringMod_btn->setActiveGraphic(
 			PLUGIN_NAME::getIconPixmap( "ringred" ) );
@@ -649,7 +650,7 @@ sidInstrumentView::sidInstrumentView( instrument * _instrument,
 			.arg( syncRingWidth[i] ) );
 
 		pixmapButton * filter_btn = new pixmapButton( this, NULL );
-		filter_btn->setCheckable( TRUE );
+		filter_btn->setCheckable( true );
 		filter_btn->move( 191, 131 + i*50 );
 		filter_btn->setActiveGraphic(
 			PLUGIN_NAME::getIconPixmap( "filterred" ) );
@@ -662,7 +663,7 @@ sidInstrumentView::sidInstrumentView( instrument * _instrument,
 			"it." ).arg( i+1 ) );
 
 		pixmapButton * test_btn = new pixmapButton( this, NULL );
-		test_btn->setCheckable( TRUE );
+		test_btn->setCheckable( true );
 		test_btn->move( 191 +2*14, 131 + i*50 );
 		test_btn->setActiveGraphic(
 			PLUGIN_NAME::getIconPixmap( "testred" ) );
@@ -682,7 +683,7 @@ sidInstrumentView::~sidInstrumentView()
 {
 }
 
-void sidInstrumentView::updateKnobHint( void )
+void sidInstrumentView::updateKnobHint()
 {
 	sidInstrument * k = castModel<sidInstrument>();
 
@@ -723,7 +724,7 @@ void sidInstrumentView::updateKnobHint( void )
 
 
 
-void sidInstrumentView::updateKnobToolTip( void )
+void sidInstrumentView::updateKnobToolTip()
 {
 	sidInstrument * k = castModel<sidInstrument>();
 	for( int i = 0; i < 3; ++i )
@@ -743,7 +744,7 @@ void sidInstrumentView::updateKnobToolTip( void )
 
 
 
-void sidInstrumentView::modelChanged( void )
+void sidInstrumentView::modelChanged()
 {
 	sidInstrument * k = castModel<sidInstrument>();
 
@@ -814,11 +815,11 @@ void sidInstrumentView::modelChanged( void )
 extern "C"
 {
 
-// neccessary for getting instance out of shared lib
-plugin * PLUGIN_EXPORT lmms_plugin_main( model *, void * _data )
+// necessary for getting instance out of shared lib
+Plugin * PLUGIN_EXPORT lmms_plugin_main( Model *, void * _data )
 {
 	return( new sidInstrument(
-				static_cast<instrumentTrack *>( _data ) ) );
+				static_cast<InstrumentTrack *>( _data ) ) );
 }
 
 
