@@ -1,7 +1,7 @@
 /*
  * vestige.cpp - instrument-plugin for hosting VST-instruments
  *
- * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -22,9 +22,9 @@
  *
  */
 
-
 #include "vestige.h"
 
+#include <QtGui/QDropEvent>
 #include <QtGui/QFileDialog>
 #include <QtGui/QMessageBox>
 #include <QtGui/QPainter>
@@ -33,22 +33,21 @@
 
 #include "engine.h"
 #include "gui_templates.h"
-#include "instrument_play_handle.h"
-#include "instrument_track.h"
-#include "vst_plugin.h"
+#include "InstrumentPlayHandle.h"
+#include "InstrumentTrack.h"
+#include "VstPlugin.h"
 #include "pixmap_button.h"
-#include "song.h"
+#include "string_pair_drag.h"
 #include "text_float.h"
 #include "tooltip.h"
 
-#undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
 
 
 extern "C"
 {
 
-plugin::descriptor vestige_plugin_descriptor =
+Plugin::Descriptor PLUGIN_EXPORT vestige_plugin_descriptor =
 {
 	STRINGIFY( PLUGIN_NAME ),
 	"VeSTige",
@@ -56,25 +55,25 @@ plugin::descriptor vestige_plugin_descriptor =
 			"VST-host for using VST(i)-plugins within LMMS" ),
 	"Tobias Doerffel <tobydox/at/users.sf.net>",
 	0x0100,
-	plugin::Instrument,
-	new pluginPixmapLoader( "logo" ),
-	NULL,
+	Plugin::Instrument,
+	new PluginPixmapLoader( "logo" ),
+	"dll",
 	NULL
 } ;
 
 }
 
 
-QPixmap * vestigeInstrumentView::s_artwork = NULL;
+QPixmap * VestigeInstrumentView::s_artwork = NULL;
 
 
-vestigeInstrument::vestigeInstrument( instrumentTrack * _instrument_track ) :
-	instrument( _instrument_track, &vestige_plugin_descriptor ),
+vestigeInstrument::vestigeInstrument( InstrumentTrack * _instrument_track ) :
+	Instrument( _instrument_track, &vestige_plugin_descriptor ),
 	m_plugin( NULL ),
 	m_pluginMutex()
 {
 	// now we need a play-handle which cares for calling play()
-	instrumentPlayHandle * iph = new instrumentPlayHandle( this );
+	InstrumentPlayHandle * iph = new InstrumentPlayHandle( this );
 	engine::getMixer()->addPlayHandle( iph );
 }
 
@@ -83,7 +82,7 @@ vestigeInstrument::vestigeInstrument( instrumentTrack * _instrument_track ) :
 
 vestigeInstrument::~vestigeInstrument()
 {
-	engine::getMixer()->removePlayHandles( getInstrumentTrack() );
+	engine::getMixer()->removePlayHandles( instrumentTrack() );
 	closePlugin();
 }
 
@@ -130,9 +129,9 @@ void vestigeInstrument::loadFile( const QString & _file )
 {
 	m_pluginMutex.lock();
 	const bool set_ch_name = ( m_plugin != NULL &&
-		getInstrumentTrack()->name() == m_plugin->name() ) ||
-			getInstrumentTrack()->name() ==
-				instrumentTrack::tr( "Default preset" );
+		instrumentTrack()->name() == m_plugin->name() ) ||
+			instrumentTrack()->name() ==
+				InstrumentTrack::tr( "Default preset" );
 	m_pluginMutex.unlock();
 
 	closePlugin();
@@ -144,7 +143,7 @@ void vestigeInstrument::loadFile( const QString & _file )
 			PLUGIN_NAME::getIconPixmap( "logo", 24, 24 ), 0 );
 
 	m_pluginMutex.lock();
-	m_plugin = new vstPlugin( m_pluginDLL );
+	m_plugin = new VstPlugin( m_pluginDLL );
 	if( m_plugin->failed() )
 	{
 		m_pluginMutex.unlock();
@@ -163,14 +162,10 @@ void vestigeInstrument::loadFile( const QString & _file )
 	}
 
 	m_plugin->showEditor();
-	connect( engine::getSong(), SIGNAL( tempoChanged( bpm_t ) ),
-			m_plugin, SLOT( setTempo( bpm_t ) ) );
-	m_plugin->setTempo( engine::getSong()->getTempo() );
-	connect( engine::getMixer(), SIGNAL( sampleRateChanged() ),
-				m_plugin, SLOT( updateSampleRate() ) );
+
 	if( set_ch_name )
 	{
-		getInstrumentTrack()->setName( m_plugin->name() );
+		instrumentTrack()->setName( m_plugin->name() );
 	}
 
 	m_pluginMutex.unlock();
@@ -196,7 +191,7 @@ void vestigeInstrument::play( sampleFrame * _buf )
 
 	const fpp_t frames = engine::getMixer()->framesPerPeriod();
 
-	getInstrumentTrack()->processAudioBuffer( _buf, frames, NULL );
+	instrumentTrack()->processAudioBuffer( _buf, frames, NULL );
 
 	m_pluginMutex.unlock();
 }
@@ -207,12 +202,15 @@ void vestigeInstrument::play( sampleFrame * _buf )
 bool vestigeInstrument::handleMidiEvent( const midiEvent & _me,
 						const midiTime & _time )
 {
-	m_pluginMutex.lock();
-	if( m_plugin != NULL )
+	if( !isMuted() )
 	{
-		m_plugin->processMidiEvent( _me, _time );
+		m_pluginMutex.lock();
+		if( m_plugin != NULL )
+		{
+			m_plugin->processMidiEvent( _me, _time );
+		}
+		m_pluginMutex.unlock();
 	}
-	m_pluginMutex.unlock();
 	return true;
 }
 
@@ -233,9 +231,9 @@ void vestigeInstrument::closePlugin( void )
 
 
 
-pluginView * vestigeInstrument::instantiateView( QWidget * _parent )
+PluginView * vestigeInstrument::instantiateView( QWidget * _parent )
 {
-	return( new vestigeInstrumentView( this, _parent ) );
+	return new VestigeInstrumentView( this, _parent );
 }
 
 
@@ -246,9 +244,9 @@ pluginView * vestigeInstrument::instantiateView( QWidget * _parent )
 
 
 
-vestigeInstrumentView::vestigeInstrumentView( instrument * _instrument,
+VestigeInstrumentView::VestigeInstrumentView( Instrument * _instrument,
 							QWidget * _parent ) :
-	instrumentView( _instrument, _parent )
+	InstrumentView( _instrument, _parent )
 {
 	if( s_artwork == NULL )
 	{
@@ -257,7 +255,7 @@ vestigeInstrumentView::vestigeInstrumentView( instrument * _instrument,
 	}
 
 	m_openPluginButton = new pixmapButton( this, "" );
-	m_openPluginButton->setCheckable( FALSE );
+	m_openPluginButton->setCheckable( false );
 	m_openPluginButton->setCursor( Qt::PointingHandCursor );
 	m_openPluginButton->move( 218, 79 );
 	m_openPluginButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
@@ -290,19 +288,21 @@ vestigeInstrumentView::vestigeInstrumentView( instrument * _instrument,
 	note_off_all_btn->setFont( pointSize<8>( note_off_all_btn->font() ) );
 	connect( note_off_all_btn, SIGNAL( clicked() ), this,
 							SLOT( noteOffAll() ) );
+
+	setAcceptDrops( true );
 }
 
 
 
 
-vestigeInstrumentView::~vestigeInstrumentView()
+VestigeInstrumentView::~VestigeInstrumentView()
 {
 }
 
 
 
 
-void vestigeInstrumentView::modelChanged( void )
+void VestigeInstrumentView::modelChanged( void )
 {
 	m_vi = castModel<vestigeInstrument>();
 }
@@ -310,7 +310,7 @@ void vestigeInstrumentView::modelChanged( void )
 
 
 
-void vestigeInstrumentView::openPlugin( void )
+void VestigeInstrumentView::openPlugin( void )
 {
 	QFileDialog ofd( NULL, tr( "Open VST-plugin" ) );
 
@@ -348,13 +348,18 @@ void vestigeInstrumentView::openPlugin( void )
 		engine::getMixer()->lock();
 		m_vi->loadFile( ofd.selectedFiles()[0] );
 		engine::getMixer()->unlock();
+		if( m_vi->m_plugin && m_vi->m_plugin->pluginWidget() )
+		{
+			m_vi->m_plugin->pluginWidget()->setWindowIcon(
+									PLUGIN_NAME::getIconPixmap( "logo" ) );
+		}
 	}
 }
 
 
 
 
-void vestigeInstrumentView::toggleGUI( void )
+void VestigeInstrumentView::toggleGUI( void )
 {
 	QMutexLocker ml( &m_vi->m_pluginMutex );
 	if( m_vi->m_plugin == NULL )
@@ -379,7 +384,7 @@ void vestigeInstrumentView::toggleGUI( void )
 
 
 
-void vestigeInstrumentView::noteOffAll( void )
+void VestigeInstrumentView::noteOffAll( void )
 {
 	m_vi->m_pluginMutex.lock();
 	if( m_vi->m_plugin != NULL )
@@ -396,7 +401,47 @@ void vestigeInstrumentView::noteOffAll( void )
 
 
 
-void vestigeInstrumentView::paintEvent( QPaintEvent * )
+void VestigeInstrumentView::dragEnterEvent( QDragEnterEvent * _dee )
+{
+	if( _dee->mimeData()->hasFormat( stringPairDrag::mimeType() ) )
+	{
+		QString txt = _dee->mimeData()->data(
+						stringPairDrag::mimeType() );
+		if( txt.section( ':', 0, 0 ) == "vstplugin" )
+		{
+			_dee->acceptProposedAction();
+		}
+		else
+		{
+			_dee->ignore();
+		}
+	}
+	else
+	{
+		_dee->ignore();
+	}
+}
+
+
+
+
+void VestigeInstrumentView::dropEvent( QDropEvent * _de )
+{
+	QString type = stringPairDrag::decodeKey( _de );
+	QString value = stringPairDrag::decodeValue( _de );
+	if( type == "vstplugin" )
+	{
+		m_vi->loadFile( value );
+		_de->accept();
+		return;
+	}
+	_de->ignore();
+}
+
+
+
+
+void VestigeInstrumentView::paintEvent( QPaintEvent * )
 {
 	QPainter p( this );
 
@@ -408,7 +453,7 @@ void vestigeInstrumentView::paintEvent( QPaintEvent * )
 					:
 				tr( "No VST-plugin loaded" );
 	QFont f = p.font();
-	f.setBold( TRUE );
+	f.setBold( true );
 	p.setFont( pointSize<10>( f ) );
 	p.setPen( QColor( 32, 160, 54 ) );
 
@@ -418,7 +463,7 @@ void vestigeInstrumentView::paintEvent( QPaintEvent * )
 	if( m_vi->m_plugin != NULL )
 	{
 		p.setPen( QColor( 251, 41, 8 ) );
-		f.setBold( FALSE );
+		f.setBold( false );
 		p.setFont( pointSize<8>( f ) );
 		p.drawText( 10, 114, tr( "by" ) + " " +
 					m_vi->m_plugin->vendorString() );
@@ -436,10 +481,10 @@ void vestigeInstrumentView::paintEvent( QPaintEvent * )
 extern "C"
 {
 
-// neccessary for getting instance out of shared lib
-plugin * lmms_plugin_main( model *, void * _data )
+// necessary for getting instance out of shared lib
+Plugin * PLUGIN_EXPORT lmms_plugin_main( Model *, void * _data )
 {
-	return( new vestigeInstrument( static_cast<instrumentTrack *>( _data ) ) );
+	return new vestigeInstrument( static_cast<InstrumentTrack *>( _data ) );
 }
 
 
