@@ -1,7 +1,7 @@
 /*
  * vestige.h - instrument VeSTige for hosting VST-plugins
  *
- * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -29,9 +29,8 @@
 
 #include <QtCore/QMutex>
 
-
-#include "instrument.h"
-#include "instrument_view.h"
+#include "Instrument.h"
+#include "InstrumentView.h"
 #include "midi.h"
 #include "note.h"
 
@@ -40,14 +39,14 @@ class QPixmap;
 class QPushButton;
 
 class pixmapButton;
-class vstPlugin;
+class VstPlugin;
 
 
-class vestigeInstrument : public instrument
+class vestigeInstrument : public Instrument
 {
 	Q_OBJECT
 public:
-	vestigeInstrument( instrumentTrack * _channel_track );
+	vestigeInstrument( InstrumentTrack * _instrument_track );
 	virtual ~vestigeInstrument();
 
 	virtual void play( sampleFrame * _working_buffer );
@@ -67,7 +66,7 @@ public:
 	virtual bool handleMidiEvent( const midiEvent & _me,
 						const midiTime & _time );
 
-	virtual pluginView * instantiateView( QWidget * _parent );
+	virtual PluginView * instantiateView( QWidget * _parent );
 
 
 private:
@@ -76,33 +75,35 @@ private:
 	int m_runningNotes[NumKeys];
 
 
-	vstPlugin * m_plugin;
+	VstPlugin * m_plugin;
 	QMutex m_pluginMutex;
 
 	QString m_pluginDLL;
 
 
-	friend class vestigeInstrumentView;
+	friend class VestigeInstrumentView;
 
 } ;
 
 
 
-class vestigeInstrumentView : public instrumentView
+class VestigeInstrumentView : public InstrumentView
 {
 	Q_OBJECT
 public:
-	vestigeInstrumentView( instrument * _instrument, QWidget * _parent );
-	virtual ~vestigeInstrumentView();
+	VestigeInstrumentView( Instrument * _instrument, QWidget * _parent );
+	virtual ~VestigeInstrumentView();
 
 
 protected slots:
 	void openPlugin( void );
-	void noteOffAll( void );
 	void toggleGUI( void );
+	void noteOffAll( void );
 
 
 protected:
+	virtual void dragEnterEvent( QDragEnterEvent * _dee );
+	virtual void dropEvent( QDropEvent * _de );
 	virtual void paintEvent( QPaintEvent * _pe );
 
 
