@@ -35,7 +35,7 @@ stringContainer::stringContainer(const float _pitch,
 {
 	for( Uint8 i = 0; i < _strings; i++ )
 	{
-		m_exists.append( FALSE );
+		m_exists.append( false );
 	}
 }
 
@@ -45,7 +45,7 @@ stringContainer::stringContainer(const float _pitch,
 void stringContainer::addString(Uint8 _harm,
 				const float _pick,
 				const float _pickup,
-				float * _impluse,
+				const float * _impulse,
 				const float _randomize,
 				const float _string_loss,
 				const float _detune,
@@ -90,7 +90,7 @@ void stringContainer::addString(Uint8 _harm,
 	m_strings.append( new vibratingString(	m_pitch * harm,
 						_pick, 
 						_pickup,
-						_impluse,
+						const_cast<float*>(_impulse),
 						m_bufferLength,
 						m_sampleRate,
 						_oversample,
@@ -98,5 +98,5 @@ void stringContainer::addString(Uint8 _harm,
 						_string_loss,
 						_detune,
 						_state ) );
-	m_exists[_id] = TRUE;
+	m_exists[_id] = true;
 }
