@@ -2,7 +2,7 @@
  * vibed.cpp - combination of PluckedStringSynth and BitInvader
  *
  * Copyright (c) 2006-2008 Danny McRae <khjklujn/at/yahoo/com>
- * 
+ *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -22,32 +22,30 @@
  *
  */
 
-
 #include <QtXml/QDomDocument>
 #include <QtCore/QMap>
 #include <QtGui/QWhatsThis>
 
 #include "vibed.h"
 #include "engine.h"
-#include "instrument_track.h"
+#include "InstrumentTrack.h"
 #include "note_play_handle.h"
 #include "tooltip.h"
 #include "base64.h"
 #include "caption_menu.h"
-#include "oscillator.h"
+#include "Oscillator.h"
 #include "string_container.h"
 #include "templates.h"
 #include "volume.h"
 #include "song.h"
 
-#undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
 
 
 extern "C"
 {
 
-plugin::descriptor PLUGIN_EXPORT vibedstrings_plugin_descriptor =
+Plugin::Descriptor PLUGIN_EXPORT vibedstrings_plugin_descriptor =
 {
 	STRINGIFY( PLUGIN_NAME ),
 	"Vibed",
@@ -55,8 +53,8 @@ plugin::descriptor PLUGIN_EXPORT vibedstrings_plugin_descriptor =
 					"Vibrating string modeler" ),
 	"Danny McRae <khjklujn/at/yahoo/com>",
 	0x0100,
-	plugin::Instrument,
-	new pluginPixmapLoader( "logo" ),
+	Plugin::Instrument,
+	new PluginPixmapLoader( "logo" ),
 	NULL,
 	NULL
 };
@@ -64,54 +62,54 @@ plugin::descriptor PLUGIN_EXPORT vibedstrings_plugin_descriptor =
 }
 
 
-vibed::vibed( instrumentTrack * instrument_track ) :
-	instrument( instrument_track, &vibedstrings_plugin_descriptor )
+vibed::vibed( InstrumentTrack * _instrumentTrack ) :
+	Instrument( _instrumentTrack, &vibedstrings_plugin_descriptor )
 {
 
-	knobModel * knob;
-	boolModel * led;
+	FloatModel * knob;
+	BoolModel * led;
 	nineButtonSelectorModel * harmonic;
 	graphModel * graphTmp;
 
 	for( Uint8 harm = 0; harm < 9; harm++ )
 	{
-		knob = new knobModel( DefaultVolume, MinVolume, MaxVolume,
+		knob = new FloatModel( DefaultVolume, MinVolume, MaxVolume,
 				1.0f, this, tr( "String %1 volume" ).arg( harm+1 ) );
 		m_volumeKnobs.append( knob );
 
-		knob = new knobModel( 0.0f, 0.0f, 0.05f, 0.001f, this, 
+		knob = new FloatModel( 0.0f, 0.0f, 0.05f, 0.001f, this,
 				tr( "String %1 stiffness" ).arg( harm+1 ) );
 		m_stiffnessKnobs.append( knob );
 
-		knob = new knobModel( 0.0f, 0.0f, 0.05f, 0.005f, this, 
+		knob = new FloatModel( 0.0f, 0.0f, 0.05f, 0.005f, this,
 				tr( "Pick %1 position" ).arg( harm+1 )  );
 		m_pickKnobs.append( knob );
 
-		knob = new knobModel( 0.05f, 0.0f, 0.05f, 0.005f, this, 
+		knob = new FloatModel( 0.05f, 0.0f, 0.05f, 0.005f, this,
 				tr( "Pickup %1 position" ).arg( harm+1 ) );
 		m_pickupKnobs.append( knob );
 
-		knob = new knobModel( 0.0f, -1.0f, 1.0f, 0.01f, this,
+		knob = new FloatModel( 0.0f, -1.0f, 1.0f, 0.01f, this,
 				tr( "Pan %1" ).arg( harm+1 )  );
 		m_panKnobs.append( knob );
 
-		knob = new knobModel( 0.0f, -0.1f, 0.1f, 0.001f, this,
+		knob = new FloatModel( 0.0f, -0.1f, 0.1f, 0.001f, this,
 				tr( "Detune %1" ).arg( harm+1 ) );
 		m_detuneKnobs.append( knob );
 
-		knob = new knobModel( 0.0f, 0.0f, 0.75f, 0.01f, this,
+		knob = new FloatModel( 0.0f, 0.0f, 0.75f, 0.01f, this,
 				tr( "Fuzziness %1 " ).arg( harm+1 ) );
 		m_randomKnobs.append( knob );
 
-		knob = new knobModel( 1, 1, 16, 1, this,
+		knob = new FloatModel( 1, 1, 16, 1, this,
 				tr( "Length %1" ).arg( harm+1 ) );
 		m_lengthKnobs.append( knob );
 
-		led = new boolModel( FALSE, this,
+		led = new BoolModel( false, this,
 				tr( "Impulse %1" ).arg( harm+1 ) );
 		m_impulses.append( led );
 
-		led = new boolModel( harm==0, this,
+		led = new BoolModel( harm==0, this,
 				tr( "Octave %1" ).arg( harm+1 )  );
 		m_powerButtons.append( led );
 
@@ -184,11 +182,11 @@ void vibed::saveSettings( QDomDocument & _doc, QDomElement & _this )
 
 			QString sampleString;
 			base64::encode(
-				(const char *)m_graphs[i]->samples(), 
-				__sampleLength * sizeof(float), 
+				(const char *)m_graphs[i]->samples(),
+				__sampleLength * sizeof(float),
 				sampleString );
 			name = "graph" + QString::number( i );
-			_this.setAttribute( name, sampleString ); 
+			_this.setAttribute( name, sampleString );
 		}
 	}
 
@@ -206,7 +204,7 @@ void vibed::loadSettings( const QDomElement & _this )
 		name = "active" + QString::number( i );
 		m_powerButtons[i]->setValue( _this.attribute( name ).toInt() );
 		
-		if( m_powerButtons[i]->value() && 
+		if( m_powerButtons[i]->value() &&
 			_this.hasAttribute( "volume" + QString::number( i ) ) )
 		{
 			name = "volume" + QString::number( i );
@@ -243,7 +241,7 @@ void vibed::loadSettings( const QDomElement & _this )
 			float * shp = 0;
 			base64::decode( _this.attribute( "graph" +
 						QString::number( i ) ),
-						(char * *) &shp,
+						&shp,
 						&size );
 			// TODO: check whether size == 128 * sizeof( float ),
 			// otherwise me might and up in a segfault
@@ -260,13 +258,13 @@ void vibed::loadSettings( const QDomElement & _this )
 		}
 	}
 	
-//	update(); 
+//	update();
 }
 
 
 
 
-QString vibed::nodeName( void ) const
+QString vibed::nodeName() const
 {
 	return( vibedstrings_plugin_descriptor.name );
 }
@@ -317,7 +315,7 @@ void vibed::playNote( notePlayHandle * _n, sampleFrame * _working_buffer )
 			if( ps->exists( string ) )
 			{
 				// pan: 0 -> left, 1 -> right
-				const float pan = ( 
+				const float pan = (
 					m_panKnobs[string]->value() + 1 ) /
 									2.0f;
 				const sample_t sample =
@@ -330,7 +328,7 @@ void vibed::playNote( notePlayHandle * _n, sampleFrame * _working_buffer )
 		}
 	}
 
-	getInstrumentTrack()->processAudioBuffer( _working_buffer, frames, _n );
+	instrumentTrack()->processAudioBuffer( _working_buffer, frames, _n );
 }
 
 
@@ -344,7 +342,7 @@ void vibed::deleteNotePluginData( notePlayHandle * _n )
 
 
 
-pluginView * vibed::instantiateView( QWidget * _parent )
+PluginView * vibed::instantiateView( QWidget * _parent )
 {
 	return( new vibedView( this, _parent ) );
 }
@@ -353,18 +351,18 @@ pluginView * vibed::instantiateView( QWidget * _parent )
 
 
 
-vibedView::vibedView( instrument * _instrument,
+vibedView::vibedView( Instrument * _instrument,
 				QWidget * _parent ) :
-	instrumentView( _instrument, _parent )
+	InstrumentView( _instrument, _parent )
 {
-	setAutoFillBackground( TRUE );
+	setAutoFillBackground( true );
 	QPalette pal;
 	pal.setBrush( backgroundRole(), PLUGIN_NAME::getIconPixmap(
 								"artwork" ) );
 	setPalette( pal );
 	
 	m_volumeKnob = new knob( knobBright_26, this );
-	m_volumeKnob->setVolumeKnob( TRUE );
+	m_volumeKnob->setVolumeKnob( true );
 	m_volumeKnob->move( 103, 142 );
 	m_volumeKnob->setHintText( tr( "Volume:" ) + " ", "" );
 	m_volumeKnob->setWhatsThis( tr( "The 'V' knob sets the volume "
@@ -372,9 +370,9 @@ vibedView::vibedView( instrument * _instrument,
 
 	m_stiffnessKnob = new knob( knobBright_26, this );
 	m_stiffnessKnob->move( 129, 142 );
-	m_stiffnessKnob->setHintText( tr( "String stiffness:" ) + 
+	m_stiffnessKnob->setHintText( tr( "String stiffness:" ) +
 					" ", "" );
-	m_stiffnessKnob->setWhatsThis( tr( 
+	m_stiffnessKnob->setWhatsThis( tr(
 "The 'S' knob sets the stiffness of the selected string.  The stiffness "
 "of the string affects how long the string will ring out.  The lower "
 "the setting, the longer the string will ring." ) );
@@ -383,15 +381,15 @@ vibedView::vibedView( instrument * _instrument,
 	m_pickKnob = new knob( knobBright_26, this );
 	m_pickKnob->move( 153, 142 );
 	m_pickKnob->setHintText( tr( "Pick position:" ) + " ", "" );
-	m_pickKnob->setWhatsThis( tr( 
+	m_pickKnob->setWhatsThis( tr(
 "The 'P' knob sets the position where the selected string will be 'picked'.  "
 "The lower the setting the closer the pick is to the bridge." ) );
 
 	m_pickupKnob = new knob( knobBright_26, this );
 	m_pickupKnob->move( 177, 142 );
-	m_pickupKnob->setHintText( tr( "Pickup position:" ) + 
+	m_pickupKnob->setHintText( tr( "Pickup position:" ) +
 				" ", "" );
-	m_pickupKnob->setWhatsThis( tr( 
+	m_pickupKnob->setWhatsThis( tr(
 "The 'PU' knob sets the position where the vibrations will be monitored "
 "for the selected string.  The lower the setting, the closer the "
 "pickup is to the bridge." ) );
@@ -399,32 +397,32 @@ vibedView::vibedView( instrument * _instrument,
 	m_panKnob = new knob( knobBright_26, this );
 	m_panKnob->move( 105, 187 );
 	m_panKnob->setHintText( tr( "Pan:" ) + " ", "" );
-	m_panKnob->setWhatsThis( tr( 
+	m_panKnob->setWhatsThis( tr(
 "The Pan knob determines the location of the selected string in the stereo "
 "field." ) );
 	
 	m_detuneKnob = new knob( knobBright_26, this );
 	m_detuneKnob->move( 150, 187 );
 	m_detuneKnob->setHintText( tr( "Detune:" ) + " ", "" );
-	m_detuneKnob->setWhatsThis( tr( 
+	m_detuneKnob->setWhatsThis( tr(
 "The Detune knob modifies the pitch of the selected string.  Settings less "
 "than zero will cause the string to sound flat.  Settings greater than zero "
 "will cause the string to sound sharp." ) );
 
 	m_randomKnob = new knob( knobBright_26, this );
 	m_randomKnob->move( 194, 187 );
-	m_randomKnob->setHintText( tr( "Fuzziness:" ) + 
+	m_randomKnob->setHintText( tr( "Fuzziness:" ) +
 				" ", "" );
-	m_randomKnob->setWhatsThis( tr( 
+	m_randomKnob->setWhatsThis( tr(
 "The Slap knob adds a bit of fuzz to the selected string which is most "
 "apparent during the attack, though it can also be used to make the string "
 "sound more 'metallic'.") );
 
 	m_lengthKnob = new knob( knobBright_26, this );
 	m_lengthKnob->move( 23, 193 );
-	m_lengthKnob->setHintText( tr( "Length:" ) + 
+	m_lengthKnob->setHintText( tr( "Length:" ) +
 				" ", "" );
-	m_lengthKnob->setWhatsThis( tr( 
+	m_lengthKnob->setWhatsThis( tr(
 "The Length knob sets the length of the selected string.  Longer strings "
 "will both ring longer and sound brighter, however, they will also eat up "
 "more CPU cycles." ) );
@@ -433,7 +431,7 @@ vibedView::vibedView( instrument * _instrument,
 	m_impulse->move( 23, 94 );
 	toolTip::add( m_impulse,
 		      tr( "Impulse or initial state" ) );
-	m_impulse->setWhatsThis( tr( 
+	m_impulse->setWhatsThis( tr(
 "The 'Imp' selector determines whether the waveform in the graph is to be "
 "treated as an impulse imparted to the string by the pick or the initial "
 "state of the string." ) );
@@ -462,7 +460,7 @@ vibedView::vibedView( instrument * _instrument,
 		this );
 
 	m_harmonic->setAccessibleName( tr( "Octave" ) );
-	m_harmonic->setWhatsThis( tr( 
+	m_harmonic->setWhatsThis( tr(
 "The Octave selector is used to choose which harmonic of the note the "
 "string will ring at.  For example, '-2' means the string will ring two "
 "octaves below the fundamental, 'F' means the string will ring at the "
@@ -499,7 +497,7 @@ vibedView::vibedView( instrument * _instrument,
 	m_graph->move( 76, 21 );
 	m_graph->resize(132, 104);
 
-	m_graph->setWhatsThis( tr( 
+	m_graph->setWhatsThis( tr(
 "The waveform editor provides control over the initial state or impulse "
 "that is used to start the string vibrating.  The buttons to the right of "
 "the graph will initialize the waveform to the selected type.  The '?' "
@@ -513,7 +511,7 @@ vibedView::vibedView( instrument * _instrument,
 "The 'N' button will normalize the waveform.") );
 	
 
-	setWhatsThis( tr( 
+	setWhatsThis( tr(
 "Vibed models up to nine independently vibrating strings.  The 'String' "
 "selector allows you to choose which string is being edited.  The 'Imp' " "selector chooses whether the graph represents an impulse or the initial "
 "state of the string.  The 'Octave' selector chooses which harmonic the "
@@ -543,9 +541,9 @@ vibedView::vibedView( instrument * _instrument,
 	
 	// String selector is not a part of the model
 	m_stringSelector->setAccessibleName( tr( "String" ) );
-	m_stringSelector->setWhatsThis( tr( 
+	m_stringSelector->setWhatsThis( tr(
 "The String selector is used to choose which string the controls are "
-"editting.  A Vibed instrument can contain up to nine independently "
+"editing.  A Vibed instrument can contain up to nine independently "
 "vibrating strings.  The LED in the lower right corner of the "
 "waveform editor indicates whether the selected string is active." ) );
 
@@ -563,8 +561,8 @@ vibedView::vibedView( instrument * _instrument,
 	toolTip::add( m_sinWaveBtn,
 				tr( "Use a sine-wave for "
 				    "current oscillator." ) );
-	connect( m_sinWaveBtn, SIGNAL (clicked ( void ) ),
-			this, SLOT ( sinWaveClicked( void ) ) );
+	connect( m_sinWaveBtn, SIGNAL (clicked () ),
+			this, SLOT ( sinWaveClicked() ) );
 
 	
 	m_triangleWaveBtn = new pixmapButton( this, tr( "Triangle wave" ) );
@@ -576,7 +574,7 @@ vibedView::vibedView( instrument * _instrument,
 	toolTip::add( m_triangleWaveBtn,
 			tr( "Use a triangle-wave "
 			    "for current oscillator." ) );
-	connect( m_triangleWaveBtn, SIGNAL ( clicked ( void ) ),
+	connect( m_triangleWaveBtn, SIGNAL ( clicked () ),
 			this, SLOT ( triangleWaveClicked( ) ) );
 
 	
@@ -589,8 +587,8 @@ vibedView::vibedView( instrument * _instrument,
 	toolTip::add( m_sawWaveBtn,
 				tr( "Use a saw-wave for "
 				    "current oscillator." ) );
-	connect( m_sawWaveBtn, SIGNAL (clicked ( void ) ),
-			this, SLOT ( sawWaveClicked( void ) ) );
+	connect( m_sawWaveBtn, SIGNAL (clicked () ),
+			this, SLOT ( sawWaveClicked() ) );
 
 	
 	m_sqrWaveBtn = new pixmapButton( this, tr( "Square wave" ) );
@@ -602,8 +600,8 @@ vibedView::vibedView( instrument * _instrument,
 	toolTip::add( m_sqrWaveBtn,
 			tr( "Use a square-wave for "
 			    "current oscillator." ) );
-	connect( m_sqrWaveBtn, SIGNAL ( clicked ( void ) ),
-			this, SLOT ( sqrWaveClicked( void ) ) );
+	connect( m_sqrWaveBtn, SIGNAL ( clicked () ),
+			this, SLOT ( sqrWaveClicked() ) );
 
 	
 	m_whiteNoiseWaveBtn = new pixmapButton( this, tr( "White noise wave" ) );
@@ -615,8 +613,8 @@ vibedView::vibedView( instrument * _instrument,
 	toolTip::add( m_whiteNoiseWaveBtn,
 			tr( "Use white-noise for "
 			    "current oscillator." ) );
-	connect( m_whiteNoiseWaveBtn, SIGNAL ( clicked ( void ) ),
-			this, SLOT ( noiseWaveClicked( void ) ) );
+	connect( m_whiteNoiseWaveBtn, SIGNAL ( clicked () ),
+			this, SLOT ( noiseWaveClicked() ) );
 
 	
 	m_usrWaveBtn = new pixmapButton( this, tr( "User defined wave" ) );
@@ -628,8 +626,8 @@ vibedView::vibedView( instrument * _instrument,
 	toolTip::add( m_usrWaveBtn,
 			tr( "Use a user-defined "
 			    "waveform for current oscillator." ) );
-	connect( m_usrWaveBtn, SIGNAL ( clicked ( void ) ),
-			this, SLOT ( usrWaveClicked( void ) ) );
+	connect( m_usrWaveBtn, SIGNAL ( clicked () ),
+			this, SLOT ( usrWaveClicked() ) );
 
 
 	m_smoothBtn = new pixmapButton( this, tr( "Smooth" ) );
@@ -638,11 +636,11 @@ vibedView::vibedView( instrument * _instrument,
 			"smooth_active" ) );
 	m_smoothBtn->setInactiveGraphic( PLUGIN_NAME::getIconPixmap(
 			"smooth_inactive" ) );
-	m_smoothBtn->setChecked( FALSE );
+	m_smoothBtn->setChecked( false );
 	toolTip::add( m_smoothBtn,
 			tr( "Click here to smooth waveform." ) );
-	connect( m_smoothBtn, SIGNAL ( clicked ( void ) ),
-			this, SLOT ( smoothClicked( void ) ) );
+	connect( m_smoothBtn, SIGNAL ( clicked () ),
+			this, SLOT ( smoothClicked() ) );
 	
 	m_normalizeBtn = new pixmapButton( this, tr( "Normalize" ) );
 	m_normalizeBtn->move( 96, 129 );
@@ -650,19 +648,19 @@ vibedView::vibedView( instrument * _instrument,
 			"normalize_active" ) );
 	m_normalizeBtn->setInactiveGraphic( PLUGIN_NAME::getIconPixmap(
 			"normalize_inactive" ) );
-	m_normalizeBtn->setChecked( FALSE );
+	m_normalizeBtn->setChecked( false );
 	toolTip::add( m_normalizeBtn,
 			tr( "Click here to normalize waveform." ) );
 
-	connect( m_normalizeBtn, SIGNAL ( clicked ( void ) ),
-			this, SLOT ( normalizeClicked( void ) ) );
+	connect( m_normalizeBtn, SIGNAL ( clicked () ),
+			this, SLOT ( normalizeClicked() ) );
 
 }
 
 
 
 
-void vibedView::modelChanged( void )
+void vibedView::modelChanged()
 {
 	showString( 0 );
 }
@@ -692,7 +690,7 @@ void vibedView::showString( Uint8 _string )
 
 
 
-void vibedView::sinWaveClicked( void )
+void vibedView::sinWaveClicked()
 {
 	m_graph->model()->setWaveToSine();
 	engine::getSong()->setModified();
@@ -700,7 +698,7 @@ void vibedView::sinWaveClicked( void )
 
 
 
-void vibedView::triangleWaveClicked( void )
+void vibedView::triangleWaveClicked()
 {
 	m_graph->model()->setWaveToTriangle();
 	engine::getSong()->setModified();
@@ -708,7 +706,7 @@ void vibedView::triangleWaveClicked( void )
 
 
 
-void vibedView::sawWaveClicked( void )
+void vibedView::sawWaveClicked()
 {
 	m_graph->model()->setWaveToSaw();
 	engine::getSong()->setModified();
@@ -716,7 +714,7 @@ void vibedView::sawWaveClicked( void )
 
 
 
-void vibedView::sqrWaveClicked( void )
+void vibedView::sqrWaveClicked()
 {
 	m_graph->model()->setWaveToSquare();
 	engine::getSong()->setModified();
@@ -724,7 +722,7 @@ void vibedView::sqrWaveClicked( void )
 
 
 
-void vibedView::noiseWaveClicked( void )
+void vibedView::noiseWaveClicked()
 {
 	m_graph->model()->setWaveToNoise();
 	engine::getSong()->setModified();
@@ -732,7 +730,7 @@ void vibedView::noiseWaveClicked( void )
 
 
 
-void vibedView::usrWaveClicked( void )
+void vibedView::usrWaveClicked()
 {
 	// TODO: load file
 	//m_graph->model()->setWaveToUser();
@@ -741,7 +739,7 @@ void vibedView::usrWaveClicked( void )
 
 
 
-void vibedView::smoothClicked( void )
+void vibedView::smoothClicked()
 {
 	m_graph->model()->smooth();
 	engine::getSong()->setModified();
@@ -749,7 +747,7 @@ void vibedView::smoothClicked( void )
 
 
 
-void vibedView::normalizeClicked( void )
+void vibedView::normalizeClicked()
 {
 	m_graph->model()->normalize();
 	engine::getSong()->setModified();
@@ -771,7 +769,7 @@ void vibedView::contextMenuEvent( QContextMenuEvent * )
 
 
 
-void vibedView::displayHelp( void )
+void vibedView::displayHelp()
 {
 	QWhatsThis::showText( mapToGlobal( rect().bottomRight() ),
 					whatsThis() );
@@ -781,10 +779,10 @@ void vibedView::displayHelp( void )
 extern "C"
 {
 
-// neccessary for getting instance out of shared lib
-plugin * PLUGIN_EXPORT lmms_plugin_main( model *, void * _data )
+// necessary for getting instance out of shared lib
+Plugin * PLUGIN_EXPORT lmms_plugin_main( Model *, void * _data )
 {
-	return( new vibed( static_cast<instrumentTrack *>( _data ) ) );
+	return( new vibed( static_cast<InstrumentTrack *>( _data ) ) );
 }
 
 
