@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_mac.cxx 6758 2009-04-13 07:32:01Z matt $"
+// "$Id: Fl_mac.cxx 7351 2010-03-29 10:35:00Z matt $"
 //
 // MacOS specific code for the Fast Light Tool Kit (FLTK).
 //
@@ -44,7 +44,7 @@
 //          Quartz calls in the near future.
 
 // FIXME moving away from Carbon, I am replacing the Scrap manager calls with Pasteboard
-//       calls that support utf8 encoding. As soon as these unction haven proven working
+//       calls that support utf8 encoding. As soon as these function haven proven working
 //       the Scrap manager calls should be removed
 #define USE_PASTEBOARD 1
 
@@ -1496,10 +1496,10 @@ void fl_open_display() {
     beenHereDoneThat = 1;
     
     FlushEvents(everyEvent,0);
-
+    
     MoreMasters(); // \todo Carbon suggests MoreMasterPointers()
     AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)QuitAppleEventHandler), 0, false );
-
+    
     // create the Mac Handle for the default cursor (a pointer to a pointer)
     GetQDGlobalsArrow(&default_cursor);
     default_cursor_ptr = &default_cursor;
@@ -1508,12 +1508,12 @@ void fl_open_display() {
     ClearMenuBar();
     AppendResMenu( GetMenuHandle( 1 ), 'DRVR' );
     DrawMenuBar();
-
+    
     // bring the application into foreground without a 'CARB' resource
     Boolean same_psn;
     ProcessSerialNumber cur_psn, front_psn;
     if( !GetCurrentProcess( &cur_psn ) && !GetFrontProcess( &front_psn ) &&
-        !SameProcess( &front_psn, &cur_psn, &same_psn ) && !same_psn )
+       !SameProcess( &front_psn, &cur_psn, &same_psn ) && !same_psn )
     {
       // only transform the application type for unbundled apps
       CFBundleRef bundle = CFBundleGetMainBundle();
@@ -1522,35 +1522,16 @@ void fl_open_display() {
       	FSRef execFs;
       	CFURLRef execUrl = CFBundleCopyExecutableURL( bundle );
       	CFURLGetFSRef( execUrl, &execFs );
-
+        
       	FSRef bundleFs;
       	GetProcessBundleLocation( &cur_psn, &bundleFs );
-
+        
       	if( !FSCompareFSRefs( &execFs, &bundleFs ) )
           bundle = NULL;
-
+        
         CFRelease(execUrl);
       }
-
-    // imm: keycode handler stub setting - use Gestalt to determine the running system version,
-    // then set the keycode_function pointer accordingly
-    SInt32 MacVersion;
-    if (Gestalt(gestaltSystemVersion, &MacVersion) == noErr)
-    {
-//      SInt32 maj, min, fix;
-//      Gestalt(gestaltSystemVersionMajor, &maj);   // e.g. 10
-//      Gestalt(gestaltSystemVersionMinor, &min);   // e.g.  4
-//      Gestalt(gestaltSystemVersionBugFix, &fix);  // e.g. 11
-      if(MacVersion >= 0x1050) { // 10.5.0 or later
-        keycode_function = keycodeToUnicode;
-      }
-      else {
-        keycode_function = keycode_wrap_old; // pre-10.5 mechanism
-      }
-    }
-    // else our default handler will be used (keycode_wrap_old)
-
-
+      
       if( !bundle )
       {
         // Earlier versions of this code tried to use weak linking, however it
@@ -1558,19 +1539,30 @@ void fl_open_display() {
         // both TransformProcessType and CPSEnableForegroundOperation, the following
         // conditional code compiled on 10.2 will still work on newer releases...
         OSErr err;
-
+        
 #if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_2
         if (TransformProcessType != NULL) {
-	  err = TransformProcessType(&cur_psn, kProcessTransformToForegroundApplication);
-	} else
+          err = TransformProcessType(&cur_psn, kProcessTransformToForegroundApplication);
+        } else
 #endif // MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_2
-        err = CPSEnableForegroundOperation(&cur_psn, 0x03, 0x3C, 0x2C, 0x1103);
-
+          err = CPSEnableForegroundOperation(&cur_psn, 0x03, 0x3C, 0x2C, 0x1103);
+        
         if (err == noErr) {
-	  SetFrontProcess( &cur_psn );
+          SetFrontProcess( &cur_psn );
         }
       }
     }
+    
+    // imm: keycode handler stub setting - use Gestalt to determine the running system version,
+    // then set the keycode_function pointer accordingly
+    keycode_function = keycode_wrap_old; // default to pre-10.5 mechanism
+    SInt32 MacVersion;
+    if (Gestalt(gestaltSystemVersion, &MacVersion) == noErr)
+    {
+      if(MacVersion >= 0x1050) { // 10.5.0 or later
+        keycode_function = keycodeToUnicode;
+      }
+    }
   }
 }
 
@@ -2185,7 +2177,7 @@ void Fl_X::make(Fl_Window* w)
       wp += 2*bx;
       hp += 2*by+bt;
     }
-    if (!(w->flags() & Fl_Window::FL_FORCE_POSITION)) {
+    if (!(w->flags() & Fl_Widget::FORCE_POSITION)) {
       // use the Carbon functions below for default window positioning
       w->x(xyPos+Fl::x());
       w->y(xyPos+Fl::y());
@@ -2232,7 +2224,7 @@ void Fl_X::make(Fl_Window* w)
       SetWindowClass(x->xid, kFloatingWindowClass);
       SetWindowActivationScope(x->xid, kWindowActivationScopeAll);
     }
-    if (!(w->flags() & Fl_Window::FL_FORCE_POSITION))
+    if (!(w->flags() & Fl_Widget::FORCE_POSITION))
     {
       WindowRef pw = Fl_X::first ? Fl_X::first->xid : 0 ;
       if (w->modal()) {
@@ -2412,7 +2404,7 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
   int is_a_resize = (W != w() || H != h());
 //  printf("Fl_Winodw::resize(X=%d, Y=%d, W=%d, H=%d), is_a_resize=%d, resize_from_system=%p, this=%p\n",
 //         X, Y, W, H, is_a_resize, resize_from_system, this);
-  if (X != x() || Y != y()) set_flag(FL_FORCE_POSITION);
+  if (X != x() || Y != y()) set_flag(FORCE_POSITION);
   else if (!is_a_resize) return;
   if ( (resize_from_system!=this) && (!parent()) && shown()) {
     if (is_a_resize) {
@@ -2450,10 +2442,8 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
  */
 void Fl_Window::make_current() 
 {
-#ifdef __APPLE_QUARTZ__
   OSStatus err;
   Fl_X::q_release_context();
-#endif
   if ( !fl_window_region )
     fl_window_region = NewRgn();
   fl_window = i->xid;
@@ -2487,29 +2477,26 @@ void Fl_Window::make_current()
     DisposeRgn( r );
   }
  
-#ifdef __APPLE_QUARTZ__
   err = QDBeginCGContext(GetWindowPort(i->xid), &i->gc);
   if (err!=noErr) 
     fprintf(stderr, "Error %d in QDBeginCGContext\n", (int)err);
   fl_gc = i->gc;
   CGContextSaveGState(fl_gc);
   Fl_X::q_fill_context();
-#if defined(USE_CAIRO) && defined (__APPLE_QUARTZ__)
+#if defined(USE_CAIRO)
    if (Fl::cairo_autolink_context()) Fl::cairo_make_current(this); // capture gc changes automatically to update the cairo context adequately
 #endif
 
-#endif
   fl_clip_region( 0 );
   SetPortClipRegion( GetWindowPort(i->xid), fl_window_region );
 
-#if defined(__APPLE_QUARTZ__) && defined(USE_CAIRO)
+#if defined(USE_CAIRO)
   // update the cairo_t context
   if (Fl::cairo_autolink_context()) Fl::cairo_make_current(this);
 #endif
 }
 
 // helper function to manage the current CGContext fl_gc
-#ifdef __APPLE_QUARTZ__
 extern Fl_Color fl_color_;
 extern class Fl_Font_Descriptor *fl_fontsize;
 extern void fl_font(class Fl_Font_Descriptor*);
@@ -2553,7 +2540,7 @@ void Fl_X::q_release_context(Fl_X *x) {
       fprintf(stderr, "Error %d in QDEndCGContext\n", (int)err);
   }
   fl_gc = 0;
-#if defined(USE_CAIRO) && defined (__APPLE_QUARTZ__)
+#if defined(USE_CAIRO)
   if (Fl::cairo_autolink_context()) Fl::cairo_make_current((Fl_Window*) 0); // capture gc changes automatically to update the cairo context adequately
 #endif
 }
@@ -2577,8 +2564,6 @@ void Fl_X::q_end_image() {
   CGContextRestoreGState(fl_gc);
 }
 
-#endif
-
 ////////////////////////////////////////////////////////////////
 // Copy & Paste fltk implementation.
 ////////////////////////////////////////////////////////////////
@@ -2881,5 +2866,5 @@ void MacUnmapWindow(Fl_Window *w, WindowPtr p) {
 #endif // FL_DOXYGEN
 
 //
-// End of "$Id: Fl_mac.cxx 6758 2009-04-13 07:32:01Z matt $".
+// End of "$Id: Fl_mac.cxx 7351 2010-03-29 10:35:00Z matt $".
 //
