@@ -1,12 +1,12 @@
 /*
   ZynAddSubFX - a software synthesizer
- 
+
   NULLMidiIn.h - a dummy Midi port
   Copyright (C) 2002-2005 Nasca Octavian Paul
   Author: Nasca Octavian Paul
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of version 2 of the GNU General Public License 
+  it under the terms of version 2 of the GNU General Public License
   as published by the Free Software Foundation.
 
   This program is distributed in the hope that it will be useful,
@@ -26,15 +26,25 @@
 #include "MidiIn.h"
 
 
-class NULLMidiIn:public MidiIn{
+/**a dummy Midi port*/
+class NULLMidiIn:public MidiIn
+{
     public:
-	NULLMidiIn();
-	~NULLMidiIn();
-	void getmidicmd(MidiCmdType &cmdtype,unsigned char &cmdchan,unsigned char *cmdparams);
+        /**Dummy Constructor
+         * \todo see if the default constructor would work here*/
+        NULLMidiIn();
+        /**Dummy Destructor
+         * \todo see if the default destructor would work here*/
+        ~NULLMidiIn();
+        /**Get the midi command,channel and parameters
+         * It returns MidiNull because it is a dummy driver
+         */
+        void getmidicmd(MidiCmdType &cmdtype,
+                        unsigned char &cmdchan,
+                        int *cmdparams);
 
     private:
 };
 
-
 #endif
 
