@@ -3,6 +3,7 @@
  *                      of ladspa plugins
  *
  * Copyright (c) 2005-2008 Danny McRae <khjklujn@netscape.net>
+ * Copyright (c) 2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -23,7 +24,7 @@
  *
  */
 
-
+#include <QtCore/QDebug>
 #include <QtCore/QDir>
 #include <QtCore/QFileInfo>
 #include <QtCore/QLibrary>
@@ -38,7 +39,7 @@
 ladspaManager::ladspaManager()
 {
 	QStringList ladspaDirectories = QString( getenv( "LADSPA_PATH" ) ).
-								split( ',' );
+								split( LADSPA_PATH_SEPERATOR );
 	ladspaDirectories += configManager::inst()->ladspaDir().split( ',' );
 
 	ladspaDirectories.push_back( configManager::inst()->pluginDir() + "ladspa" );
@@ -83,6 +84,10 @@ ladspaManager::ladspaManager()
 							f.fileName() );
 				}
 			}
+			else
+			{
+				qWarning() << plugin_lib.errorString();
+			}
 		}
 	}
 	
