@@ -26,7 +26,7 @@
 #include "sample_record_handle.h"
 #include "bb_track.h"
 #include "engine.h"
-#include "instrument_track.h"
+#include "InstrumentTrack.h"
 #include "pattern.h"
 #include "sample_buffer.h"
 #include "sample_track.h"
@@ -84,7 +84,7 @@ void sampleRecordHandle::play( sampleFrame * /*_working_buffer*/ )
 
 
 
-bool sampleRecordHandle::done( void ) const
+bool sampleRecordHandle::done() const
 {
 	return false;
 }
@@ -100,7 +100,7 @@ bool sampleRecordHandle::isFromTrack( const track * _track ) const
 
 
 
-f_cnt_t sampleRecordHandle::framesRecorded( void ) const
+f_cnt_t sampleRecordHandle::framesRecorded() const
 {
 	return( m_framesRecorded );
 }
