@@ -1,8 +1,8 @@
 /*
- * song.cpp - root of the model-tree
+ * song.cpp - root of the model tree
+ *
+ * Copyright (c) 2004-2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
- * Copyright (c) 2004-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
  *
  */
 
-
+#include <QtCore/QCoreApplication>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 #include <QtGui/QFileDialog>
@@ -31,8 +31,8 @@
 #include <math.h>
 
 #include "song.h"
-#include "automation_track.h"
-#include "automation_editor.h"
+#include "AutomationTrack.h"
+#include "AutomationEditor.h"
 #include "bb_editor.h"
 #include "bb_track.h"
 #include "bb_track_container.h"
@@ -40,21 +40,21 @@
 #include "ControllerRackView.h"
 #include "ControllerConnection.h"
 #include "embed.h"
-#include "envelope_and_lfo_parameters.h"
+#include "EnvelopeAndLfoParameters.h"
 #include "export_project_dialog.h"
-#include "fx_mixer.h"
-#include "fx_mixer_view.h"
-#include "import_filter.h"
-#include "instrument_track.h"
-#include "main_window.h"
-#include "midi_client.h"
+#include "FxMixer.h"
+#include "FxMixerView.h"
+#include "ImportFilter.h"
+#include "InstrumentTrack.h"
+#include "MainWindow.h"
+#include "MidiClient.h"
 #include "mmp.h"
 #include "note_play_handle.h"
 #include "pattern.h"
 #include "piano_roll.h"
-#include "project_journal.h"
+#include "ProjectJournal.h"
 #include "project_notes.h"
-#include "project_renderer.h"
+#include "ProjectRenderer.h"
 #include "rename_dialog.h"
 #include "song_editor.h"
 #include "templates.h"
@@ -66,9 +66,9 @@ tick_t midiTime::s_ticksPerTact = DefaultTicksPerTact;
 
 
 
-song::song( void ) :
+song::song() :
 	trackContainer(),
-	m_globalAutomationTrack( dynamic_cast<automationTrack *>(
+	m_globalAutomationTrack( dynamic_cast<AutomationTrack *>(
 				track::create( track::HiddenAutomationTrack,
 								this ) ) ),
 	m_tempoModel( DefaultTempo, MinTempo, MaxTempo, this, tr( "Tempo" ) ),
@@ -123,7 +123,7 @@ song::~song()
 
 
 
-void song::masterVolumeChanged( void )
+void song::masterVolumeChanged()
 {
 	engine::getMixer()->setMasterGain( m_masterVolumeModel.value() /
 								100.0f );
@@ -132,7 +132,7 @@ void song::masterVolumeChanged( void )
 
 
 
-void song::setTempo( void )
+void song::setTempo()
 {
 	const bpm_t tempo = (bpm_t) m_tempoModel.value();
 	engine::getMixer()->lock();
@@ -156,7 +156,7 @@ void song::setTempo( void )
 
 
 
-void song::setTimeSignature( void )
+void song::setTimeSignature()
 {
 	midiTime::setTicksPerTact( ticksPerTact() );
 	emit timeSignatureChanged( m_oldTicksPerTact, ticksPerTact() );
@@ -167,7 +167,7 @@ void song::setTimeSignature( void )
 
 
 
-void song::doActions( void )
+void song::doActions()
 {
 	while( !m_actions.empty() )
 	{
@@ -281,7 +281,7 @@ void song::doActions( void )
 
 
 
-void song::processNextBuffer( void )
+void song::processNextBuffer()
 {
 	doActions();
 
@@ -300,7 +300,7 @@ void song::processNextBuffer( void )
 			// at song-start we have to reset the LFOs
 			if( m_playPos[Mode_PlaySong] == 0 )
 			{
-				envelopeAndLFOParameters::resetLFO();
+				EnvelopeAndLfoParameters::instances()->reset();
 			}
 			break;
 
@@ -338,7 +338,7 @@ void song::processNextBuffer( void )
 		return;
 	}
 
-	// check for looping-mode and act if neccessary
+	// check for looping-mode and act if necessary
 	timeLine * tl = m_playPos[m_playMode].m_timeLine;
 	bool check_loop = tl != NULL && m_exporting == false &&
 				tl->loopPointsEnabled() &&
@@ -467,7 +467,7 @@ void song::processNextBuffer( void )
 
 
 
-bool song::realTimeTask( void ) const
+bool song::realTimeTask() const
 {
 	return !( m_exporting == true || ( m_playMode == Mode_PlayPattern &&
 		  	m_patternToPlay != NULL &&
@@ -477,7 +477,7 @@ bool song::realTimeTask( void ) const
 
 
 
-void song::play( void )
+void song::play()
 {
 	m_recording = false;
 	if( m_playing == true )
@@ -500,7 +500,7 @@ void song::play( void )
 
 
 
-void song::record( void )
+void song::record()
 {
 	m_recording = true;
 	// TODO: Implement
@@ -509,7 +509,7 @@ void song::record( void )
 
 
 
-void song::playAndRecord( void )
+void song::playAndRecord()
 {
 	play();
 	m_recording = true;
@@ -532,7 +532,7 @@ void song::playTrack( track * _trackToPlay )
 
 
 
-void song::playBB( void )
+void song::playBB()
 {
 	if( m_playing == true )
 	{
@@ -561,7 +561,7 @@ void song::playPattern( pattern * _patternToPlay, bool _loop )
 
 
 
-void song::updateLength( void )
+void song::updateLength()
 {
 	m_length = 0;
 	m_tracksMutex.lockForRead();
@@ -591,7 +591,7 @@ void song::setPlayPos( tick_t _ticks, PlayModes _play_mode )
 
 
 
-void song::stop( void )
+void song::stop()
 {
 	m_actions.push_back( ActionStop );
 }
@@ -601,7 +601,7 @@ void song::stop( void )
 
 
 
-void song::pause( void )
+void song::pause()
 {
 	m_actions.push_back( ActionPause );
 }
@@ -609,7 +609,7 @@ void song::pause( void )
 
 
 
-void song::resumeFromPause( void )
+void song::resumeFromPause()
 {
 	m_actions.push_back( ActionResumeFromPause );
 }
@@ -617,7 +617,7 @@ void song::resumeFromPause( void )
 
 
 
-void song::startExport( void )
+void song::startExport()
 {
 	stop();
 	doActions();
@@ -631,7 +631,7 @@ void song::startExport( void )
 
 
 
-void song::stopExport( void )
+void song::stopExport()
 {
 	stop();
 	m_exporting = false;
@@ -640,7 +640,7 @@ void song::stopExport( void )
 
 
 
-void song::insertBar( void )
+void song::insertBar()
 {
 	m_tracksMutex.lockForRead();
 	for( trackList::const_iterator it = tracks().begin();
@@ -654,7 +654,7 @@ void song::insertBar( void )
 
 
 
-void song::removeBar( void )
+void song::removeBar()
 {
 	m_tracksMutex.lockForRead();
 	for( trackList::const_iterator it = tracks().begin();
@@ -668,7 +668,7 @@ void song::removeBar( void )
 
 
 
-void song::addBBTrack( void )
+void song::addBBTrack()
 {
 	engine::getMixer()->lock();
 	track * t = track::create( track::BBTrack, this );
@@ -680,7 +680,7 @@ void song::addBBTrack( void )
 
 
 
-void song::addSampleTrack( void )
+void song::addSampleTrack()
 {
 	engine::getMixer()->lock();
 	(void) track::create( track::SampleTrack, this );
@@ -690,7 +690,7 @@ void song::addSampleTrack( void )
 
 
 
-void song::addAutomationTrack( void )
+void song::addAutomationTrack()
 {
 	engine::getMixer()->lock();
 	(void) track::create( track::AutomationTrack, this );
@@ -700,7 +700,7 @@ void song::addAutomationTrack( void )
 
 
 
-bpm_t song::getTempo( void )
+bpm_t song::getTempo()
 {
 	return (bpm_t) m_tempoModel.value();
 }
@@ -708,17 +708,17 @@ bpm_t song::getTempo( void )
 
 
 
-automationPattern * song::tempoAutomationPattern( void )
+AutomationPattern * song::tempoAutomationPattern()
 {
-	return automationPattern::globalAutomationPattern( &m_tempoModel );
+	return AutomationPattern::globalAutomationPattern( &m_tempoModel );
 }
 
 
 
 
-void song::clearProject( void )
+void song::clearProject()
 {
-	engine::getProjectJournal()->setJournalling( false );
+	engine::projectJournal()->setJournalling( false );
 
 	if( m_playing )
 	{
@@ -734,24 +734,30 @@ void song::clearProject( void )
 	{
 		engine::getSongEditor()->clearAllTracks();
 	}
-	if( engine::getFxMixerView() )
+	if( engine::fxMixerView() )
 	{
-		engine::getFxMixerView()->clear();
+		engine::fxMixerView()->clear();
 	}
 	QCoreApplication::sendPostedEvents();
 	engine::getBBTrackContainer()->clearAllTracks();
 	clearAllTracks();
 
-	engine::getFxMixer()->clear();
+	engine::fxMixer()->clear();
 
-	if( engine::getAutomationEditor() )
+	if( engine::automationEditor() )
 	{
-		engine::getAutomationEditor()->setCurrentPattern( NULL );
+		engine::automationEditor()->setCurrentPattern( NULL );
 	}
-	automationPattern::globalAutomationPattern( &m_tempoModel )->clear();
-	automationPattern::globalAutomationPattern( &m_masterVolumeModel )->
+
+	m_tempoModel.reset();
+	m_masterVolumeModel.reset();
+	m_masterPitchModel.reset();
+	m_timeSigModel.reset();
+
+	AutomationPattern::globalAutomationPattern( &m_tempoModel )->clear();
+	AutomationPattern::globalAutomationPattern( &m_masterVolumeModel )->
 									clear();
-	automationPattern::globalAutomationPattern( &m_masterPitchModel )->
+	AutomationPattern::globalAutomationPattern( &m_masterPitchModel )->
 									clear();
 
 	engine::getMixer()->unlock();
@@ -769,9 +775,9 @@ void song::clearProject( void )
 
 	emit dataChanged();
 
-	engine::getProjectJournal()->clearJournal();
+	engine::projectJournal()->clearJournal();
 
-	engine::getProjectJournal()->setJournalling( true );
+	engine::projectJournal()->setJournalling( true );
 }
 
 
@@ -779,7 +785,7 @@ void song::clearProject( void )
 
 
 // create new file
-void song::createNewProject( void )
+void song::createNewProject()
 {
 	QString default_template = configManager::inst()->userProjectsDir()
 						+ "templates/default.mpt";
@@ -801,17 +807,17 @@ void song::createNewProject( void )
 
 	clearProject();
 
-	engine::getProjectJournal()->setJournalling( false );
+	engine::projectJournal()->setJournalling( false );
 
 	m_fileName = m_oldFileName = "";
 
 	track * t;
 	t = track::create( track::InstrumentTrack, this );
-	dynamic_cast<instrumentTrack * >( t )->loadInstrument(
+	dynamic_cast<InstrumentTrack * >( t )->loadInstrument(
 					"tripleoscillator" );
 	t = track::create( track::InstrumentTrack,
 						engine::getBBTrackContainer() );
-	dynamic_cast<instrumentTrack * >( t )->loadInstrument(
+	dynamic_cast<InstrumentTrack * >( t )->loadInstrument(
 						"tripleoscillator" );
 	track::create( track::SampleTrack, this );
 	track::create( track::BBTrack, this );
@@ -828,15 +834,15 @@ void song::createNewProject( void )
 
 	engine::getBBTrackContainer()->updateAfterTrackAdd();
 
-	engine::getProjectJournal()->setJournalling( true );
+	engine::projectJournal()->setJournalling( true );
 
 	QCoreApplication::sendPostedEvents();
 
 	m_modified = false;
 
-	if( engine::getMainWindow() )
+	if( engine::mainWindow() )
 	{
-		engine::getMainWindow()->resetWindowTitle();
+		engine::mainWindow()->resetWindowTitle();
 	}
 }
 
@@ -850,9 +856,9 @@ void song::createNewProjectFromTemplate( const QString & _template )
 	// saving...
 	m_fileName = m_oldFileName = "";
 	// update window title
-	if( engine::getMainWindow() )
+	if( engine::mainWindow() )
 	{
-		engine::getMainWindow()->resetWindowTitle();
+		engine::mainWindow()->resetWindowTitle();
 	}
 
 }
@@ -867,7 +873,7 @@ void song::loadProject( const QString & _file_name )
 
 	clearProject();
 
-	engine::getProjectJournal()->setJournalling( false );
+	engine::projectJournal()->setJournalling( false );
 
 	m_fileName = _file_name;
 	m_oldFileName = _file_name;
@@ -907,18 +913,16 @@ void song::loadProject( const QString & _file_name )
 		{
 			if( node.nodeName() == "trackcontainer" )
 			{
-				( (journallingObject *)( this ) )->
+				( (JournallingObject *)( this ) )->
 					restoreState( node.toElement() );
 			}
 			else if( node.nodeName() == "controllers" )
 			{
 				restoreControllerStates( node.toElement() );
 			}
-			else if( node.nodeName() ==
-					engine::getFxMixer()->nodeName() )
+			else if( node.nodeName() == engine::fxMixer()->nodeName() )
 			{
-				engine::getFxMixer()->restoreState(
-							node.toElement() );
+				engine::fxMixer()->restoreState( node.toElement() );
 			}
 			else if( engine::hasGUI() )
 			{
@@ -935,10 +939,10 @@ void song::loadProject( const QString & _file_name )
 							node.toElement() );
 				}
 				else if( node.nodeName() ==
-					engine::getAutomationEditor()->
+					engine::automationEditor()->
 								nodeName() )
 				{
-					engine::getAutomationEditor()->
+					engine::automationEditor()->
 						restoreState( node.toElement() );
 				}
 				else if( node.nodeName() ==
@@ -946,7 +950,7 @@ void song::loadProject( const QString & _file_name )
 								nodeName() )
 				{
 					 engine::getProjectNotes()->
-			serializingObject::restoreState( node.toElement() );
+			SerializingObject::restoreState( node.toElement() );
 				}
 				else if( node.nodeName() ==
 						m_playPos[Mode_PlaySong].
@@ -970,33 +974,29 @@ void song::loadProject( const QString & _file_name )
 	ControllerConnection::finalizeConnections();
 
 	// resolve all IDs so that autoModels are automated
-	automationPattern::resolveAllIDs();
+	AutomationPattern::resolveAllIDs();
 
 
 	engine::getMixer()->unlock();
 
 	configManager::inst()->addRecentlyOpenedProject( _file_name );
 
-	engine::getProjectJournal()->setJournalling( true );
+	engine::projectJournal()->setJournalling( true );
+
+	emit projectLoaded();
 
 	m_loadingProject = false;
 	m_modified = false;
 
-	if( engine::getMainWindow() )
+	if( engine::mainWindow() )
 	{
-		engine::getMainWindow()->resetWindowTitle();
-	}
-	if( engine::getSongEditor() )
-	{
-		engine::getSongEditor()->scrolled( 0 );
+		engine::mainWindow()->resetWindowTitle();
 	}
 }
 
 
-
-
-// save current song
-bool song::saveProject( void )
+// only save current song as _filename and do nothing else
+bool song::saveProjectFile( const QString & _filename )
 {
 	multimediaProject mmp( multimediaProject::SongProject );
 
@@ -1005,26 +1005,34 @@ bool song::saveProject( void )
 	m_masterVolumeModel.saveSettings( mmp, mmp.head(), "mastervol" );
 	m_masterPitchModel.saveSettings( mmp, mmp.head(), "masterpitch" );
 
-
 	saveState( mmp, mmp.content() );
 
 	m_globalAutomationTrack->saveState( mmp, mmp.content() );
-	engine::getFxMixer()->saveState( mmp, mmp.content() );
+	engine::fxMixer()->saveState( mmp, mmp.content() );
 	if( engine::hasGUI() )
 	{
 		engine::getControllerRackView()->saveState( mmp, mmp.content() );
 		engine::getPianoRoll()->saveState( mmp, mmp.content() );
-		engine::getAutomationEditor()->saveState( mmp, mmp.content() );
+		engine::automationEditor()->saveState( mmp, mmp.content() );
 		engine::getProjectNotes()->
-			serializingObject::saveState( mmp, mmp.content() );
+			SerializingObject::saveState( mmp, mmp.content() );
 		m_playPos[Mode_PlaySong].m_timeLine->saveState(
 							mmp, mmp.content() );
 	}
 
 	saveControllerStates( mmp, mmp.content() );
 
+    return mmp.writeFile( _filename );
+}
+
+
+
+// save current song and update the gui
+bool song::guiSaveProject()
+{
+	multimediaProject mmp( multimediaProject::SongProject );
 	m_fileName = mmp.nameWithExtension( m_fileName );
-	if( mmp.writeFile( m_fileName ) == true && engine::hasGUI() )
+	if( saveProjectFile( m_fileName ) && engine::hasGUI() )
 	{
 		textFloat::displayMessage( tr( "Project saved" ),
 					tr( "The project %1 is now saved."
@@ -1033,7 +1041,7 @@ bool song::saveProject( void )
 									2000 );
 		configManager::inst()->addRecentlyOpenedProject( m_fileName );
 		m_modified = false;
-		engine::getMainWindow()->resetWindowTitle();
+		engine::mainWindow()->resetWindowTitle();
 	}
 	else if( engine::hasGUI() )
 	{
@@ -1051,12 +1059,12 @@ bool song::saveProject( void )
 
 
 // save current song in given filename
-bool song::saveProjectAs( const QString & _file_name )
+bool song::guiSaveProjectAs( const QString & _file_name )
 {
 	QString o = m_oldFileName;
 	m_oldFileName = m_fileName;
 	m_fileName = _file_name;
-	if( saveProject() == false )
+	if( guiSaveProject() == false )
 	{
 		m_fileName = m_oldFileName;
 		m_oldFileName = o;
@@ -1069,15 +1077,22 @@ bool song::saveProjectAs( const QString & _file_name )
 
 
 
-void song::importProject( void )
+void song::importProject()
 {
 	QFileDialog ofd( NULL, tr( "Import file" ),
 			configManager::inst()->userProjectsDir(),
-			tr("MIDI sequences (*.mid *.rif);;FL Studio projects (*.flp)") );
+			tr("MIDI sequences") +
+			" (*.mid *.midi *.rmi);;" +
+			tr("FL Studio projects") +
+			" (*.flp"
+			 	");;" +
+			tr("All file types") +
+			" (*.*)");
+
 	ofd.setFileMode( QFileDialog::ExistingFiles );
 	if( ofd.exec () == QDialog::Accepted && !ofd.selectedFiles().isEmpty() )
 	{
-		importFilter::import( ofd.selectedFiles()[0], this );
+		ImportFilter::import( ofd.selectedFiles()[0], this );
 	}
 }
 
@@ -1112,11 +1127,11 @@ void song::restoreControllerStates( const QDomElement & _this )
 
 
 
-void song::exportProject( void )
+void song::exportProject()
 {
 	if( isEmpty() )
 	{
-		QMessageBox::information( engine::getMainWindow(),
+		QMessageBox::information( engine::mainWindow(),
 				tr( "Empty project" ),
 				tr( "This project is empty so exporting makes "
 					"no sense. Please put some items into "
@@ -1124,13 +1139,13 @@ void song::exportProject( void )
 		return;
 	}
 
-	QFileDialog efd( engine::getMainWindow() );
+	QFileDialog efd( engine::mainWindow() );
 	efd.setFileMode( QFileDialog::AnyFile );
 	efd.setAcceptMode( QFileDialog::AcceptSave );
 	int idx = 0;
 	QStringList types;
 	while( __fileEncodeDevices[idx].m_fileFormat !=
-					projectRenderer::NumFileFormats )
+					ProjectRenderer::NumFileFormats )
 	{
 		types << tr( __fileEncodeDevices[idx].m_description );
 		++idx;
@@ -1138,7 +1153,7 @@ void song::exportProject( void )
 	efd.setFilters( types );
 
 	QString base_filename;
-	if( m_fileName != "" )
+	if( !m_fileName.isEmpty() )
 	{
 		efd.setDirectory( QFileInfo( m_fileName ).absolutePath() );
 		base_filename = QFileInfo( m_fileName ).completeBaseName();
@@ -1152,11 +1167,11 @@ void song::exportProject( void )
 	efd.setWindowTitle( tr( "Select file for project-export..." ) );
 
 	if( efd.exec() == QDialog::Accepted &&
-		!efd.selectedFiles().isEmpty() && efd.selectedFiles()[0] != "" )
+		!efd.selectedFiles().isEmpty() && !efd.selectedFiles()[0].isEmpty() )
 	{
 		const QString export_file_name = efd.selectedFiles()[0];
 		exportProjectDialog epd( export_file_name,
-						engine::getMainWindow() );
+						engine::mainWindow() );
 		epd.exec();
 	}
 }
@@ -1164,7 +1179,7 @@ void song::exportProject( void )
 
 
 
-void song::updateFramesPerTick( void )
+void song::updateFramesPerTick()
 {
 	engine::updateFramesPerTick();
 }
@@ -1172,16 +1187,16 @@ void song::updateFramesPerTick( void )
 
 
 
-void song::setModified( void )
+void song::setModified()
 {
 	if( !m_loadingProject )
 	{
 		m_modified = true;
-		if( engine::getMainWindow() &&
+		if( engine::mainWindow() &&
 			QThread::currentThread() ==
-					engine::getMainWindow()->thread() )
+					engine::mainWindow()->thread() )
 		{
-			engine::getMainWindow()->resetWindowTitle();
+			engine::mainWindow()->resetWindowTitle();
 		}
 	}
 }
