@@ -3,8 +3,8 @@
 /*
  * main_window.cpp - implementation of LMMS-main-window
  *
- * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * 
+ * Copyright (c) 2004-2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -46,6 +46,8 @@
 #include "embed.h"
 #include "engine.h"
 #include "FxMixerView.h" 
+#include "InstrumentTrack.h"
+#include "PianoView.h"
 #include "about_dialog.h"
 #include "ControllerRackView.h"
 #include "file_browser.h"
@@ -60,7 +62,7 @@
 #include "ToolPlugin.h"
 #include "tool_button.h"
 #include "ProjectJournal.h"
-#include "automation_editor.h"
+#include "AutomationEditor.h"
 #include "templates.h"
 
 
@@ -70,7 +72,8 @@ MainWindow::MainWindow( void ) :
 	m_workspace( NULL ),
 	m_templatesMenu( NULL ),
 	m_recentlyOpenedProjectsMenu( NULL ),
-	m_toolsMenu( NULL )
+	m_toolsMenu( NULL ),
+	m_autoSaveTimer( this )
 {
 	setAttribute( Qt::WA_DeleteOnClose );
 
@@ -180,6 +183,10 @@ MainWindow::MainWindow( void ) :
 
 
 	m_updateTimer.start( 1000 / 20, this );	// 20 fps
+
+	// connect auto save
+	connect(&m_autoSaveTimer, SIGNAL(timeout()), this, SLOT(autoSave()));
+	m_autoSaveTimer.start(1000 * 60); // 1 minute
 }
 
 
@@ -475,10 +482,8 @@ void MainWindow::finalize( void )
 	m_toolBarLayout->addWidget( fx_mixer_window, 1, 5 );
 	m_toolBarLayout->addWidget( project_notes_window, 1, 6 );
 	m_toolBarLayout->addWidget( controllers_window, 1, 7 );
-	m_toolBarLayout->addWidget( controllers_window, 1, 7 );
 	m_toolBarLayout->setColumnStretch( 100, 1 );
 
-
 	// setup-dialog opened before?
 	if( !configManager::inst()->value( "app", "configured" ).toInt() )
 	{
@@ -726,7 +731,7 @@ bool MainWindow::saveProject( void )
 	}
 	else
 	{
-		engine::getSong()->saveProject();
+		engine::getSong()->guiSaveProject();
 	}
 	return( TRUE );
 }
@@ -755,7 +760,7 @@ bool MainWindow::saveProjectAs( void )
 	if( sfd.exec () == QFileDialog::Accepted &&
 		!sfd.selectedFiles().isEmpty() && sfd.selectedFiles()[0] != "" )
 	{
-		engine::getSong()->saveProjectAs(
+		engine::getSong()->guiSaveProjectAs(
 						sfd.selectedFiles()[0] );
 		return( TRUE );
 	}
@@ -856,7 +861,7 @@ void MainWindow::togglePianoRollWin( void )
 
 void MainWindow::toggleAutomationEditorWin( void )
 {
-	toggleWindow( engine::getAutomationEditor() );
+	toggleWindow( engine::automationEditor() );
 }
 
 
@@ -898,6 +903,9 @@ void MainWindow::closeEvent( QCloseEvent * _ce )
 {
 	if( mayChangeProject() )
 	{
+		// delete recovery file
+		QDir working(configManager::inst()->workingDir());
+		working.remove("recover.mmp");
 		_ce->accept();
 	}
 	else
@@ -928,7 +936,18 @@ void MainWindow::keyPressEvent( QKeyEvent * _ke )
 		case Qt::Key_Shift: m_keyMods.m_shift = TRUE; break;
 		case Qt::Key_Alt: m_keyMods.m_alt = TRUE; break;
 		default:
-			QMainWindow::keyPressEvent( _ke );
+		{
+			InstrumentTrackWindow * w =
+						InstrumentTrackView::topLevelInstrumentTrackWindow();
+			if( w )
+			{
+				w->pianoView()->keyPressEvent( _ke );
+			}
+			if( !_ke->isAccepted() )
+			{
+				QMainWindow::keyPressEvent( _ke );
+			}
+		}
 	}
 }
 
@@ -943,14 +962,22 @@ void MainWindow::keyReleaseEvent( QKeyEvent * _ke )
 		case Qt::Key_Shift: m_keyMods.m_shift = FALSE; break;
 		case Qt::Key_Alt: m_keyMods.m_alt = FALSE; break;
 		default:
-			QMainWindow::keyReleaseEvent( _ke );
+			if( InstrumentTrackView::topLevelInstrumentTrackWindow() )
+			{
+				InstrumentTrackView::topLevelInstrumentTrackWindow()->
+					pianoView()->keyReleaseEvent( _ke );
+			}
+			if( !_ke->isAccepted() )
+			{
+				QMainWindow::keyReleaseEvent( _ke );
+			}
 	}
 }
 
 
 
 
-void MainWindow::timerEvent( QTimerEvent * )
+void MainWindow::timerEvent( QTimerEvent * _te)
 {
 	emit periodicUpdate();
 }
@@ -1018,6 +1045,13 @@ void MainWindow::browseHelp( void )
 
 
 
+void MainWindow::autoSave()
+{
+	QDir work(configManager::inst()->workingDir());
+	engine::getSong()->saveProjectFile(work.absoluteFilePath("recover.mmp"));
+}
+
+
 #include "moc_MainWindow.cxx"
 
 
