@@ -2,7 +2,7 @@
  * file_browser.cpp - implementation of the project-, preset- and
  *                    sample-file-browser
  *
- * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -39,10 +39,10 @@
 #include "embed.h"
 #include "engine.h"
 #include "gui_templates.h"
-#include "import_filter.h"
-#include "instrument.h"
-#include "instrument_track.h"
-#include "main_window.h"
+#include "ImportFilter.h"
+#include "Instrument.h"
+#include "InstrumentTrack.h"
+#include "MainWindow.h"
 #include "mmp.h"
 #include "preset_preview_play_handle.h"
 #include "sample_play_handle.h"
@@ -63,7 +63,7 @@ enum TreeWidgetItemTypes
 fileBrowser::fileBrowser( const QString & _directories, const QString & _filter,
 			const QString & _title, const QPixmap & _pm,
 			QWidget * _parent, bool _dirs_as_items ) :
-	sideBarWidget( _title, _pm, _parent ),
+	SideBarWidget( _title, _pm, _parent ),
 	m_directories( _directories ),
 	m_filter( _filter ),
 	m_dirsAsItems( _dirs_as_items )
@@ -435,8 +435,9 @@ void fileBrowserTreeWidget::mousePressEvent( QMouseEvent * _me )
 			m_previewPlayHandle = s;
 			delete tf;
 		}
-		else if( f->handling() == fileItem::LoadAsPreset ||
-				f->handling() == fileItem::LoadByPlugin )
+		else if( f->type() != fileItem::VstPluginFile &&
+				( f->handling() == fileItem::LoadAsPreset ||
+				f->handling() == fileItem::LoadByPlugin ) )
 		{
 			m_previewPlayHandle =
 				new presetPreviewPlayHandle( f->fullName(),
@@ -473,30 +474,28 @@ void fileBrowserTreeWidget::mouseMoveEvent( QMouseEvent * _me )
 			switch( f->type() )
 			{
 				case fileItem::PresetFile:
-					new stringPairDrag( "presetfile",
-								f->fullName(),
-							embed::getIconPixmap(
-								"preset_file" ),
-									this );
+	new stringPairDrag( f->handling() == fileItem::LoadAsPreset ?
+					"presetfile" : "pluginpresetfile",
+				f->fullName(),
+				embed::getIconPixmap( "preset_file" ), this );
 					break;
 
 				case fileItem::SampleFile:
-					new stringPairDrag( "samplefile",
-								f->fullName(),
-							embed::getIconPixmap(
-								"sample_file" ),
-									this );
+	new stringPairDrag( "samplefile", f->fullName(),
+				embed::getIconPixmap( "sample_file" ), this );
 					break;
 
 				case fileItem::MidiFile:
 // don't allow dragging FLP-files as FLP import filter clears project
 // without asking
 //				case fileItem::FlpFile:
-					new stringPairDrag( "importedproject",
-								f->fullName(),
-							embed::getIconPixmap(
-								"midi_file" ),
-									this );
+	new stringPairDrag( "importedproject", f->fullName(),
+				embed::getIconPixmap( "midi_file" ), this );
+					break;
+
+				case fileItem::VstPluginFile:
+	new stringPairDrag( "vstplugin", f->fullName(),
+				embed::getIconPixmap( "sample_file" ), this );
 					break;
 
 				default:
@@ -543,13 +542,13 @@ void fileBrowserTreeWidget::mouseReleaseEvent( QMouseEvent * _me )
 
 
 
-void fileBrowserTreeWidget::handleFile( fileItem * f, instrumentTrack * _it )
+void fileBrowserTreeWidget::handleFile( fileItem * f, InstrumentTrack * _it )
 {
 	engine::getMixer()->lock();
 	switch( f->handling() )
 	{
 		case fileItem::LoadAsProject:
-			if( engine::getMainWindow()->mayChangeProject() )
+			if( engine::mainWindow()->mayChangeProject() )
 			{
 				engine::getSong()->loadProject( f->fullName() );
 			}
@@ -558,9 +557,9 @@ void fileBrowserTreeWidget::handleFile( fileItem * f, instrumentTrack * _it )
 		case fileItem::LoadByPlugin:
 		{
 			const QString e = f->extension();
-			instrument * i = _it->getInstrument();
+			Instrument * i = _it->instrument();
 			if( i == NULL ||
-				!i->getDescriptor()->supportsFileType( e ) )
+				!i->descriptor()->supportsFileType( e ) )
 			{
 				i = _it->loadInstrument(
 					engine::pluginFileHandling()[e] );
@@ -572,7 +571,7 @@ void fileBrowserTreeWidget::handleFile( fileItem * f, instrumentTrack * _it )
 		case fileItem::LoadAsPreset:
 		{
 			multimediaProject mmp( f->fullName() );
-			instrumentTrack::removeMidiPortNode( mmp );
+			InstrumentTrack::removeMidiPortNode( mmp );
 			_it->setSimpleSerializing();
 			_it->loadSettings( mmp.content().toElement() );
 			break;
@@ -580,11 +579,11 @@ void fileBrowserTreeWidget::handleFile( fileItem * f, instrumentTrack * _it )
 
 		case fileItem::ImportAsProject:
 			if( f->type() == fileItem::FlpFile &&
-				!engine::getMainWindow()->mayChangeProject() )
+				!engine::mainWindow()->mayChangeProject() )
 			{
 				break;
 			}
-			importFilter::import( f->fullName(),
+			ImportFilter::import( f->fullName(),
 							engine::getSong() );
 			break;
 
@@ -616,7 +615,7 @@ void fileBrowserTreeWidget::activateListItem( QTreeWidgetItem * _item,
 	else if( f->handling() != fileItem::NotSupported )
 	{
 		engine::getMixer()->lock();
-		instrumentTrack * it = dynamic_cast<instrumentTrack *>(
+		InstrumentTrack * it = dynamic_cast<InstrumentTrack *>(
 				track::create( track::InstrumentTrack,
 					engine::getBBTrackContainer() ) );
 		handleFile( f, it );
@@ -633,7 +632,7 @@ void fileBrowserTreeWidget::openInNewInstrumentTrack( trackContainer * _tc )
 		m_contextMenuItem->handling() == fileItem::LoadByPlugin )
 	{
 		engine::getMixer()->lock();
-		instrumentTrack * it = dynamic_cast<instrumentTrack *>(
+		InstrumentTrack * it = dynamic_cast<InstrumentTrack *>(
 				track::create( track::InstrumentTrack, _tc ) );
 		handleFile( m_contextMenuItem, it );
 		engine::getMixer()->unlock();
@@ -663,7 +662,7 @@ void fileBrowserTreeWidget::sendToActiveInstrumentTrack( void )
 {
 	// get all windows opened in the workspace
 	QList<QMdiSubWindow*> pl =
-			engine::getMainWindow()->workspace()->
+			engine::mainWindow()->workspace()->
 				subWindowList( QMdiArea::StackingOrder );
 	QListIterator<QMdiSubWindow *> w( pl );
 	w.toBack();
@@ -671,8 +670,8 @@ void fileBrowserTreeWidget::sendToActiveInstrumentTrack( void )
 	// instrument-track
 	while( w.hasPrevious() )
 	{
-		instrumentTrackWindow * itw =
-			dynamic_cast<instrumentTrackWindow *>(
+		InstrumentTrackWindow * itw =
+			dynamic_cast<InstrumentTrackWindow *>(
 						w.previous()->widget() );
 		if( itw != NULL && itw->isHidden() == false )
 		{
@@ -992,6 +991,11 @@ void fileItem::determineFileType( void )
 		m_type = FlpFile;
 		m_handling = ImportAsProject;
 	}
+	else if( ext == "dll" )
+	{
+		m_type = VstPluginFile;
+		m_handling = LoadByPlugin;
+	}
 	else
 	{
 		m_type = UnknownFile;
