@@ -2,7 +2,7 @@
  * bb_track.cpp - implementation of class bbTrack and bbTCO
  *
  * Copyright (c) 2004-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * 
+ *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -22,13 +22,11 @@
  *
  */
 
-
 #include <QtXml/QDomElement>
 #include <QtGui/QColorDialog>
 #include <QtGui/QMenu>
 #include <QtGui/QPainter>
 
-
 #include "bb_editor.h"
 #include "bb_track.h"
 #include "bb_track_container.h"
@@ -55,7 +53,7 @@ bbTCO::bbTCO( track * _track, unsigned int _color ) :
 					bbTrack::numOfBBTrack( getTrack() ) );
 	if( t > 0 )
 	{
-		saveJournallingState( FALSE );
+		saveJournallingState( false );
 		changeLength( midiTime( t, 0 ) );
 		restoreJournallingState();
 	}
@@ -179,7 +177,7 @@ void bbTCOView::paintEvent( QPaintEvent * )
 	{
 		col = QColor( 160, 160, 160 );
 	}
-	if( isSelected() == TRUE )
+	if( isSelected() == true )
 	{
 		col = QColor( qMax( col.red() - 128, 0 ),
 					qMax( col.green() - 128, 0 ), 255 );
@@ -223,7 +221,7 @@ void bbTCOView::paintEvent( QPaintEvent * )
 
 
 
-void bbTCOView::openInBBEditor( void )
+void bbTCOView::openInBBEditor()
 {
 	engine::getBBTrackContainer()->setCurrentBB( bbTrack::numOfBBTrack(
 							m_bbTCO->getTrack() ) );
@@ -234,7 +232,7 @@ void bbTCOView::openInBBEditor( void )
 
 
 
-void bbTCOView::resetName( void )
+void bbTCOView::resetName()
 {
 	m_bbTCO->setName( m_bbTCO->getTrack()->name() );
 }
@@ -242,7 +240,7 @@ void bbTCOView::resetName( void )
 
 
 
-void bbTCOView::changeName( void )
+void bbTCOView::changeName()
 {
 	QString s = m_bbTCO->name();
 	renameDialog rename_dlg( s );
@@ -253,7 +251,7 @@ void bbTCOView::changeName( void )
 
 
 
-void bbTCOView::changeColor( void )
+void bbTCOView::changeColor()
 {
 	QColor _new_color = QColorDialog::getColor( m_bbTCO->m_color );
 	if( !_new_color.isValid() )
@@ -348,7 +346,7 @@ bool bbTrack::play( const midiTime & _start, const fpp_t _frames,
 {
 	if( isMuted() )
 	{
-		return( FALSE );
+		return( false );
 	}
 
 	if( _tco_num >= 0 )
@@ -364,7 +362,7 @@ bool bbTrack::play( const midiTime & _start, const fpp_t _frames,
 
 	if( tcos.size() == 0 )
 	{
-		return( FALSE );
+		return( false );
 	}
 
 	midiTime lastPosition;
@@ -387,7 +385,7 @@ bool bbTrack::play( const midiTime & _start, const fpp_t _frames,
 							_offset,
 							s_infoMap[this] ) );
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -432,7 +430,7 @@ void bbTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 			_this.parentNode().parentNode().nodeName() != "clone" &&
 			_this.parentNode().nodeName() != "journaldata" )
 	{
-		( (journallingObject *)( engine::getBBTrackContainer() ) )->
+		( (JournallingObject *)( engine::getBBTrackContainer() ) )->
 						saveState( _doc, _this );
 	}
 	if( _this.parentNode().parentNode().nodeName() == "clone" )
@@ -458,12 +456,16 @@ void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 		engine::getBBTrackContainer()->createTCOsForBB( dst );
 		trackContainer::trackList tl =
 					engine::getBBTrackContainer()->tracks();
+		// copy TCOs of all tracks from source BB (at bar "src") to destination
+		// TCOs (which are created if they do not exist yet)
 		for( trackContainer::trackList::iterator it = tl.begin();
 							it != tl.end(); ++it )
 		{
 			( *it )->getTCO( src )->copy();
 			( *it )->getTCO( dst )->paste();
 		}
+		setName( tr( "Clone of %1" ).arg(
+					_this.parentNode().toElement().attribute( "name" ) ) );
 	}
 	else
 	{
@@ -471,7 +473,7 @@ void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 					trackContainer::classNodeName() );
 		if( node.isElement() )
 		{
-			( (journallingObject *)engine::getBBTrackContainer() )->
+			( (JournallingObject *)engine::getBBTrackContainer() )->
 					restoreState( node.toElement() );
 		}
 	}
@@ -544,7 +546,7 @@ bbTrackView::bbTrackView( bbTrack * _bbt, trackContainerView * _tcv ) :
 	setFixedHeight( 32 );
 	// drag'n'drop with bb-tracks only causes troubles (and makes no sense
 	// too), so disable it
-	setAcceptDrops( FALSE );
+	setAcceptDrops( false );
 
 	m_trackLabel = new trackLabelButton( this, getTrackSettingsWidget() );
 	m_trackLabel->setIcon( embed::getIconPixmap( "bb_track" ) );
@@ -566,7 +568,7 @@ bbTrackView::~bbTrackView()
 
 
 
-bool bbTrackView::close( void )
+bool bbTrackView::close()
 {
 	engine::getBBEditor()->removeBBView( bbTrack::s_infoMap[m_bbTrack] );
 	return( trackView::close() );
@@ -575,7 +577,7 @@ bool bbTrackView::close( void )
 
 
 
-void bbTrackView::clickedTrackLabel( void )
+void bbTrackView::clickedTrackLabel()
 {
 	engine::getBBTrackContainer()->setCurrentBB(
 					bbTrack::numOfBBTrack( m_bbTrack ) );
